"""
Utility functions to scale samples for distribution comparison.

Felix Metzner 2021
"""
import numpy as np
import pandas as pd

from dataclasses import dataclass
from collections.abc import MutableMapping
from typing import Tuple, List, Dict, Any, Iterator, Iterable, ClassVar

from templatefitter.plotter.histogram_variable import HistVariable

from rdstar.offline_analysis.reco_modes.reco_mode_utilities import RecoModeCollection


__all__ = [
    "MCPreScaler",
    "RecoModeScaleInfo",
    "SampleScaleInfo",
]


@dataclass(frozen=True)
class MCPreScaler:
    scaling_info: Dict[Tuple[int, ...], float]
    category_col_name: str
    original_weight_column_name: str
    only_relative_scaling: bool

    def __post_init__(self) -> None:
        assert all(isinstance(v, float) and v >= 0.0 for v in self.scaling_info.values()), self.scaling_info

    def apply_scaling(
        self,
        df: pd.DataFrame,
        new_weight_column_name: str,
    ) -> None:
        assert self.category_col_name in df.columns, self.category_col_name
        assert self.original_weight_column_name in df.columns, self.original_weight_column_name

        assert new_weight_column_name not in df.columns, new_weight_column_name

        df.loc[:, new_weight_column_name] = df[self.original_weight_column_name]

        for category_values, scaling_value in self.scaling_info.items():
            cond: pd.Series = df[self.category_col_name].isin(category_values)
            df.loc[cond, new_weight_column_name] *= scaling_value

        if self.only_relative_scaling:
            original_sum_of_weights: float = df[self.original_weight_column_name].sum()
            new_sum_of_weights: float = df[self.original_weight_column_name].sum()

            _scaling: float = original_sum_of_weights / new_sum_of_weights
            df.loc[:, new_weight_column_name] *= _scaling


@dataclass(frozen=True)
class RecoModeScaleInfo:
    reco_mode_key: str
    reco_mode_ids: Tuple[int, ...]
    scale_factor: float


class SampleScaleInfo(MutableMapping):
    default_tmp_weight_column_name: ClassVar[str] = "__tmp_sample_rescaling_weight_col__"

    def __init__(
        self,
        weight_column_name: str,
        reco_mod_column_name: str,
        comparison_hist_var: HistVariable,
    ) -> None:
        self.weight_col_name: str = weight_column_name
        self.reco_mode_id_col: str = reco_mod_column_name
        self.comparison_hist_var: HistVariable = comparison_hist_var

        self.__scale_infos: Dict[str, RecoModeScaleInfo] = dict()

    def add_scale_info(self, reco_mode_key: str, reco_mode_ids: Tuple[int, ...], scale_factor: float) -> None:
        scale_info = RecoModeScaleInfo(
            reco_mode_key=reco_mode_key,
            reco_mode_ids=reco_mode_ids,
            scale_factor=scale_factor,
        )
        self.add_entry(scale_info=scale_info)

    def add_entry(self, scale_info: RecoModeScaleInfo) -> None:
        assert scale_info.reco_mode_key not in self, scale_info.reco_mode_key
        for id_value in scale_info.reco_mode_ids:
            assert not any(id_value in v.reco_mode_ids for v in self.__scale_infos.values()), id_value
        self.__scale_infos.update({scale_info.reco_mode_key: scale_info})

    def fill_from(
        self,
        mc_df: pd.DataFrame,
        data_df: pd.DataFrame,
        reco_mode_info_collection: RecoModeCollection,
    ) -> None:
        mc_range_condition: pd.Series = self.get_range_condition(df=mc_df)
        data_range_condition: pd.Series = self.get_range_condition(df=data_df)

        _successful_cases: List[Tuple[Any, ...]] = []
        for reco_mode_key, reco_mode_ids in reco_mode_info_collection.main_reco_mode_infos_as_tuple:
            mc_reco_mode_cond: pd.Series = mc_df[self.reco_mode_id_col].isin(reco_mode_ids)
            data_reco_mode_cond: pd.Series = data_df[self.reco_mode_id_col].isin(reco_mode_ids)

            mc_sum_of_weights: float = mc_df[mc_range_condition & mc_reco_mode_cond][self.weight_col_name].sum()
            data_event_count: int = len(data_df[data_range_condition & data_reco_mode_cond].index)

            _more_info: Tuple[Any, ...] = (
                reco_mode_key,
                reco_mode_ids,
                repr(reco_mode_info_collection),
                _successful_cases,
            )
            assert np.isfinite(data_event_count), (data_event_count, *_more_info)
            assert np.isfinite(mc_sum_of_weights), (mc_sum_of_weights, *_more_info)
            assert mc_sum_of_weights > 0, (mc_sum_of_weights, *_more_info)

            scale_factor: float = data_event_count / mc_sum_of_weights
            self.add_scale_info(
                reco_mode_key=reco_mode_key,
                reco_mode_ids=tuple(reco_mode_ids),
                scale_factor=scale_factor,
            )
            _successful_cases.append((reco_mode_key, reco_mode_ids))

    def fill_with(self, scale_infos_iterable: Iterable[RecoModeScaleInfo]) -> None:
        for scale_info in scale_infos_iterable:
            self.add_entry(scale_info=scale_info)

    def get_range_condition(self, df: pd.DataFrame) -> pd.Series:
        var_col_name: str = self.comparison_hist_var.df_label
        lower_limit, upper_limit = self.comparison_hist_var.scope  # type: float, float
        return (df[var_col_name] >= lower_limit) & (df[var_col_name] <= upper_limit)

    def rescale_sample(
        self,
        mc_sample: pd.DataFrame,
        new_weight_column_name: str,
    ) -> str:
        assert new_weight_column_name not in mc_sample.columns, new_weight_column_name
        range_condition: pd.Series = self.get_range_condition(df=mc_sample)

        mc_sample.loc[:, new_weight_column_name] = mc_sample[self.weight_col_name]

        for scale_info in self.__scale_infos.values():
            reco_mode_cond: pd.Series = mc_sample[self.reco_mode_id_col].isin(scale_info.reco_mode_ids)
            cond: pd.Series = range_condition & reco_mode_cond
            mc_sample.loc[cond, new_weight_column_name] *= scale_info.scale_factor

        return new_weight_column_name

    def rescale_sample_for_all_modes_case_only(
        self,
        mc_sample: pd.DataFrame,
        new_weight_column_name: str,
        correct_overall_normalization: bool,
    ) -> str:
        assert new_weight_column_name not in mc_sample.columns, new_weight_column_name
        range_condition: pd.Series = self.get_range_condition(df=mc_sample)

        mc_sample.loc[:, new_weight_column_name] = mc_sample[self.weight_col_name]

        rescale_info: Dict[str, float] = dict()
        for scale_info in self.__scale_infos.values():
            reco_mode_cond: pd.Series = mc_sample[self.reco_mode_id_col].isin(scale_info.reco_mode_ids)
            cond: pd.Series = range_condition & reco_mode_cond

            rescale_info.update({scale_info.reco_mode_key: mc_sample.loc[cond, self.weight_col_name].sum()})
            mc_sample.loc[cond, new_weight_column_name] *= scale_info.scale_factor

        if not correct_overall_normalization:
            all_ids: List[int] = [id_val for v in self.__scale_infos.values() for id_val in v.reco_mode_ids]
            all_modes_cond: pd.Series = mc_sample[self.reco_mode_id_col].isin(all_ids)
            original_sow: float = mc_sample.loc[range_condition & all_modes_cond, self.weight_col_name].sum()

            new_sow = sum([si.scale_factor * rescale_info[si.reco_mode_key] for si in self.__scale_infos.values()])

            inverted_norm_weight: float = original_sow / new_sow
            mc_sample.loc[range_condition & all_modes_cond, new_weight_column_name] *= inverted_norm_weight

        return new_weight_column_name

    def __iter__(self) -> Iterator[str]:
        return iter(self.__scale_infos.keys())

    def __len__(self) -> int:
        return len(self.__scale_infos)

    def __getitem__(self, reco_mode_key: str) -> RecoModeScaleInfo:
        if reco_mode_key not in self.__scale_infos.keys():
            raise KeyError(f"No scaling information for reco mode key '{reco_mode_key}' available!")
        return self.__scale_infos[reco_mode_key]

    def __setitem__(self, reco_mode_key: str, scale_info: RecoModeScaleInfo) -> None:
        assert reco_mode_key == scale_info.reco_mode_key, (reco_mode_key, scale_info.reco_mode_key)
        self.add_entry(scale_info=scale_info)

    def __delitem__(self, reco_mode_key: str) -> None:
        assert reco_mode_key in self.__scale_infos.keys()
        del self.__scale_infos[reco_mode_key]
