"""
Providing methods to combine and modify latex strings for various latex labels.

Felix Metzner 2021
"""

from typing import Union, Optional, Sequence, Tuple, List

__all__ = [
    "LatexObsNameMethods",
]


class LatexObsNameMethods:
    decay_arrow = r" \, \rightarrow \, "  # type: str

    @staticmethod
    def get_abs_of(var_str: str) -> str:
        if var_str.startswith("$"):
            new_var_str = r"$\left|" + var_str[1:]
        else:
            new_var_str = r"$\left|$" + var_str
        if new_var_str.endswith("$"):
            return new_var_str[:-1] + r"\right|$"
        else:
            return new_var_str + r"$\right|$"

    @staticmethod
    def get_delta_of(var_str: str) -> str:
        if var_str.startswith("$"):
            return r"$\Delta" + var_str[1:]
        else:
            return r"$\Delta$" + var_str

    @staticmethod
    def get_probability_of(sub: Optional[str] = None, sup: Optional[str] = None) -> str:
        var = r"$\mathcal{P}$"
        if sub is not None:
            var = LatexObsNameMethods.add_sub_script(var_str=var, script=sub)
        if sup is not None:
            var = LatexObsNameMethods.add_super_script(var_str=var, script=sup)
        return var

    @staticmethod
    def add_super_script(var_str: str, script: str) -> str:
        return LatexObsNameMethods.__add_super_or_sub_script(var_str=var_str, script=script, sup=True)

    @staticmethod
    def add_sub_script(var_str: str, script: str) -> str:
        return LatexObsNameMethods.__add_super_or_sub_script(var_str=var_str, script=script, sup=False)

    @staticmethod
    def __add_super_or_sub_script(var_str: str, script: str, sup: bool) -> str:
        sup_or_sub = r"^{" if sup else r"_{"

        bare_var = LatexObsNameMethods.get_bare_var_str(var_str=var_str)  # type: str
        bare_script = LatexObsNameMethods.get_bare_var_str(var_str=script)  # type: str

        return r"$" + bare_var + sup_or_sub + bare_script + r"}$"

    @staticmethod
    def x_in_parentheses(x: str) -> str:
        bare_x = LatexObsNameMethods.get_bare_var_str(var_str=x)  # type: str
        return r"$(" + bare_x + r")$"

    @staticmethod
    def x_in_brackets(x: str) -> str:
        bare_x = LatexObsNameMethods.get_bare_var_str(var_str=x)  # type: str
        return r"$[" + bare_x + r"]$"

    @staticmethod
    def x_in_braces(x: str) -> str:
        bare_x = LatexObsNameMethods.get_bare_var_str(var_str=x)  # type: str
        return r"$\{" + bare_x + r"\}$"

    @staticmethod
    def create_x_of_y(x: str, y: str) -> str:
        bare_x = LatexObsNameMethods.get_bare_var_str(var_str=x)  # type: str
        bare_y = LatexObsNameMethods.get_bare_var_str(var_str=y)  # type: str
        return r"$" + bare_x + r"(" + bare_y + r")$"

    @staticmethod
    def create_angle_of_x(x: str, angle_symbol: Optional[str] = None) -> str:
        bare_angle_symbol = r"\alpha"  # type: str
        # bare_angle_symbol = r"\measuredangle"  # type: str
        if angle_symbol is not None:
            bare_angle_symbol = LatexObsNameMethods.get_bare_var_str(angle_symbol)

        bare_x = LatexObsNameMethods.get_bare_var_str(var_str=x)  # type: str
        return r"$" + bare_angle_symbol + r"_" + bare_x + r"$"

    @staticmethod
    def create_x_of_nth_daughter_of_y(x: str, y: str, n: int) -> str:
        assert 0 <= n <= 20, n
        bare_x = LatexObsNameMethods.get_bare_var_str(var_str=x)  # type: str
        bare_y = LatexObsNameMethods.get_bare_var_str(var_str=y)  # type: str
        return r"$" + bare_x + r"\;(\mathrm{Daughter\;}" + rf"{n}" + "\mathrm{\:of\;}" + bare_y + r")$"  # noqa: W605

    @staticmethod
    def get_bare_var_str(var_str: str) -> str:
        if var_str.startswith("$") and var_str.endswith("$"):
            bare_var = var_str[1:-1]
        elif "$" not in var_str:
            bare_var = var_str
        else:
            raise ValueError(
                f"Parameter 'var_str' must either contain no '$', or start and end with '$', "
                f"but '{var_str}' was provided!"
            )
        return bare_var

    @staticmethod
    def get_anti_particle(particle_str: str) -> str:
        bare_particle_str = LatexObsNameMethods.get_bare_var_str(var_str=particle_str)  # type: str
        if bare_particle_str.startswith("\mathrm{"):  # noqa: W605
            p1, p2 = bare_particle_str.split(sep="}", maxsplit=1)
            return r"$\overline{" + p1 + r"}}" + p2 + r"$"  # noqa: W605
        else:
            starts_with_slash = bare_particle_str.startswith("\\")  # type: bool
            if starts_with_slash:
                bare_particle_str = bare_particle_str.split(sep="\\", maxsplit=1)[1]

            bare_rest = ""
            for separator_str in [r"_", r"^", r"(", r" ", "\\"]:
                parts = bare_particle_str.split(sep=separator_str, maxsplit=1)
                if len(parts) == 1:
                    continue
                if separator_str == "\\":
                    print(f"is double slash separator {separator_str} !")
                bare_particle_str = parts[0]
                bare_rest = separator_str + parts[1] + bare_rest

            start_str = "$\overline{\\" if starts_with_slash else r"$\overline{"  # type: str  # noqa: W605
            return start_str + bare_particle_str + r"}" + bare_rest + r"$"

    @staticmethod
    def get_bare_decay_products(to_parts: Union[str, Sequence[str]]) -> str:
        if isinstance(to_parts, str):
            bare_to_parts = LatexObsNameMethods.get_bare_var_str(var_str=to_parts)  # type: str
        elif isinstance(to_parts, Sequence):
            bare_to_parts = r" \; ".join([LatexObsNameMethods.get_bare_var_str(var_str=tp_str) for tp_str in to_parts])
        else:
            raise TypeError(
                f"Argument 'to_parts' must be string or sequence of strings,"
                f"but object of type {type(to_parts).__name__} was provided!"
            )
        return bare_to_parts

    @staticmethod
    def get_decay(from_part: str, to_parts: Union[str, Sequence[str]]) -> str:
        bare_from_part = LatexObsNameMethods.get_bare_var_str(var_str=from_part)  # type: str
        bare_to_parts = LatexObsNameMethods.get_bare_decay_products(to_parts=to_parts)
        return r"$" + bare_from_part + LatexObsNameMethods.decay_arrow + bare_to_parts + r"$"

    @staticmethod
    def get_decay_and_last_switched(from_part: str, to_parts: Union[str, Sequence[str]]) -> Tuple[str, str]:
        assert len(to_parts) > 2, to_parts
        decay_label_str = LatexObsNameMethods.get_decay(from_part=from_part, to_parts=to_parts)  # type: str

        bare_from_part = LatexObsNameMethods.get_bare_var_str(var_str=from_part)  # type: str
        to_parts_switched = list(to_parts)[:-2] + list(to_parts)[-2::][::-1]  # type: List[str]
        bare_to_parts_switched = LatexObsNameMethods.get_bare_decay_products(to_parts=to_parts_switched)  # type: str
        dec_label_w_sw_str = (
            r"$" + bare_from_part + LatexObsNameMethods.decay_arrow + bare_to_parts_switched + r"$"
        )  # type: str

        return decay_label_str, dec_label_w_sw_str

    @staticmethod
    def get_decay_products(to_parts: Union[str, Sequence[str]]) -> str:
        bare_to_parts = LatexObsNameMethods.get_bare_decay_products(to_parts=to_parts)
        return r"$" + bare_to_parts + r"$"

    @staticmethod
    def get_products_from_decay(process_label: str) -> str:
        assert process_label.startswith("$") and process_label.endswith("$"), process_label
        assert LatexObsNameMethods.decay_arrow in process_label, (process_label, LatexObsNameMethods.decay_arrow)
        bare_process_str = LatexObsNameMethods.get_bare_var_str(var_str=process_label)  # type: str
        bare_products_str = bare_process_str.split(sep=LatexObsNameMethods.decay_arrow, maxsplit=1)[1]  # type: str
        return r"$" + bare_products_str + r"$"

    @staticmethod
    def get_as_sub_decay(decay: str, with_mother: bool) -> str:
        bare_decay_str = LatexObsNameMethods.get_bare_var_str(var_str=decay)  # type: str
        mother, decay_products = bare_decay_str.split(sep=LatexObsNameMethods.decay_arrow, maxsplit=1)  # type: str, str
        to_decay_products = r"$" + LatexObsNameMethods.decay_arrow + decay_products + r"$"  # type: str

        if with_mother:
            sub_dec_str = LatexObsNameMethods.x_in_parentheses(x=to_decay_products)  # type: str
            bare_sub_dec_str = LatexObsNameMethods.get_bare_var_str(var_str=sub_dec_str)  # type: str
            return r"$" + mother + r" \; " + bare_sub_dec_str + r"$"

        return LatexObsNameMethods.x_in_parentheses(x=to_decay_products)
