from typing import Tuple, Dict

from rdstar.offline_analysis.efficiency_calculation.simple_efficiency_calculator import DataTypeFraction
from rdstar.offline_analysis.efficiency_calculation.rdstar_efficiency_definitions import (
    get_two_sided_event_fraction,
    get_one_sided_event_fraction,
    get_two_sided_event_fraction_with_two_bfs,
)

from rdstar.branching_fractions import belle_neutral_b_meson_decay_info as belle_mixed_b_bfs
from rdstar.branching_fractions import belle_charged_b_meson_decay_info as belle_charged_b_bfs
from rdstar.branching_fractions import belle_special_b_meson_decay_info as belle_special_b_bfs

from rdstar.offline_analysis.dss_fitting.utilities import dss_bf_fitting_utils as bf_fit_utils

bf_b0_dlnu = 2 * belle_mixed_b_bfs.belle_bf_b0_to_d_l_nu  # type: float
bf_bp_dlnu = 2 * belle_charged_b_bfs.belle_bf_bp_to_d_l_nu  # type: float
bf_b0_dstlnu = 2 * belle_mixed_b_bfs.belle_bf_b0_to_d_st_l_nu  # type: float
bf_bp_dstlnu = 2 * belle_charged_b_bfs.belle_bf_bp_to_d_st_l_nu  # type: float
bf_b0_dtaunu = belle_mixed_b_bfs.belle_bf_b0_to_d_tau_nu  # type: float
bf_bp_dtaunu = belle_charged_b_bfs.belle_bf_bp_to_d_tau_nu  # type: float
bf_b0_dsttaunu = belle_mixed_b_bfs.belle_bf_b0_to_d_st_tau_nu  # type: float
bf_bp_dsttaunu = belle_charged_b_bfs.belle_bf_bp_to_d_st_tau_nu  # type: float

bf_b0_dzero_st_lnu = 2 * belle_mixed_b_bfs.belle_bf_b0_to_dzero_st_l_nu  # type: float
bf_bp_dzero_st_lnu = 2 * belle_charged_b_bfs.belle_bf_bp_to_dzero_st_l_nu  # type: float
bf_b0_done_lnu = 2 * belle_mixed_b_bfs.belle_bf_b0_to_done_l_nu  # type: float
bf_bp_done_lnu = 2 * belle_charged_b_bfs.belle_bf_bp_to_done_l_nu  # type: float
bf_b0_done_prime_lnu = 2 * belle_mixed_b_bfs.belle_bf_b0_to_done_prime_l_nu  # type: float
bf_bp_done_prime_lnu = 2 * belle_charged_b_bfs.belle_bf_bp_to_done_prime_l_nu  # type: float
bf_b0_dtwo_st_lnu = 2 * belle_mixed_b_bfs.belle_bf_b0_to_dtwo_st_l_nu  # type: float
bf_bp_dtwo_st_lnu = 2 * belle_charged_b_bfs.belle_bf_bp_to_dtwo_st_l_nu  # type: float

__dss_pi_plus_factor = 0.6667  # type: float
__dss_pi_zero_factor = 0.3333  # type: float

__dss_charged_done_factor = 0.5991  # type: float
__dss_mixed_done_factor = 0.5991  # type: float

mixed_removed = (
    2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs
    + belle_mixed_b_bfs.sum_of_belle_b0_to_dss_l_nu_bfs
    + belle_special_b_bfs.sum_bf_b0_rare
    + belle_special_b_bfs.sum_bf_b0_ulnu
)  # type: float
charged_removed = (
    2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs
    + belle_charged_b_bfs.sum_of_belle_bp_to_dss_l_nu_bfs
    + belle_special_b_bfs.sum_bf_bp_rare
    + belle_special_b_bfs.sum_bf_bp_ulnu
)  # type: float

mixed_special_removed = 0.0  # type: float
charged_special_removed = 0.0  # type: float

dss_b_neutral_data_types = (
    "BZD1E",
    "BZD0sE",
    "BZDp1E",
    "BZD2sE",
    "BZD1Mu",
    "BZD0sMu",
    "BZDp1Mu",
    "BZD2sMu",
    "TBZD1Tau",
    "TBZD0sTau",
    "TBZDp1Tau",
    "TBZD2sTau",
)
dss_b_charged_data_types = (
    "BMD1E",
    "BMD0sE",
    "BMDp1E",
    "BMD2sE",
    "BMD1Mu",
    "BMD0sMu",
    "BMDp1Mu",
    "BMD2sMu",
    "TBMD1Tau",
    "TBMD0sTau",
    "TBMDp1Tau",
    "TBMD2sTau",
)
gap_b_neutral_data_types = (
    "GapD1Mixed",
    "GapDEtaMixed",
    "GapDstEtaMixed",
    "GapDPiPiMixed",
    "GapDstPiPiMixed",
)
gap_b_charged_data_types = (
    "GapD1Charged",
    "GapDEtaCharged",
    "GapDstEtaCharged",
    "GapDPiPiCharged",
    "GapDstPiPiCharged",
)
special_b_neutral_data_types = (
    "rareMixed",
    "ulnuMixed",
)
special_b_charged_data_types = (
    "rareCharged",
    "ulnuCharged",
)

# Normalisation Fractions
charged_B_to_D_st_l_nu_fractions = (
    DataTypeFraction(
        data_types=("Charged",),
        fraction=get_two_sided_event_fraction(bf=bf_bp_dstlnu, bf_removed=charged_removed),
    ),
    DataTypeFraction(
        data_types=dss_b_charged_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_bp_dstlnu, bf_removed=0),
    ),
    DataTypeFraction(
        data_types=special_b_charged_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_bp_dstlnu, bf_removed=charged_special_removed),
    ),
    DataTypeFraction(
        data_types=gap_b_charged_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_bp_dstlnu, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_D_st_l_nu_fractions = (
    DataTypeFraction(
        data_types=("Mixed",),
        fraction=get_two_sided_event_fraction(bf=bf_b0_dstlnu, bf_removed=mixed_removed),
    ),
    DataTypeFraction(
        data_types=dss_b_neutral_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_b0_dstlnu, bf_removed=0),
    ),
    DataTypeFraction(
        data_types=special_b_neutral_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_b0_dstlnu, bf_removed=mixed_special_removed),
    ),
    DataTypeFraction(
        data_types=gap_b_neutral_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_b0_dstlnu, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

charged_B_to_D_l_nu_fractions = (
    DataTypeFraction(
        data_types=("Charged",),
        fraction=get_two_sided_event_fraction(bf=bf_bp_dlnu, bf_removed=charged_removed),
    ),
    DataTypeFraction(
        data_types=dss_b_charged_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_bp_dlnu, bf_removed=0),
    ),
    DataTypeFraction(
        data_types=special_b_charged_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_bp_dlnu, bf_removed=charged_special_removed),
    ),
    DataTypeFraction(
        data_types=gap_b_charged_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_bp_dlnu, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_D_l_nu_fractions = (
    DataTypeFraction(
        data_types=("Mixed",),
        fraction=get_two_sided_event_fraction(bf=bf_b0_dlnu, bf_removed=mixed_removed),
    ),
    DataTypeFraction(
        data_types=dss_b_neutral_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_b0_dlnu, bf_removed=0),
    ),
    DataTypeFraction(
        data_types=special_b_neutral_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_b0_dlnu, bf_removed=mixed_special_removed),
    ),
    DataTypeFraction(
        data_types=gap_b_neutral_data_types,
        fraction=get_one_sided_event_fraction(bf=bf_b0_dlnu, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

# Dzerostar Fractions
charged_B_to_Dzerostar_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BMD0sE", "BMD0sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_bp_dzero_st_lnu * __dss_pi_plus_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMD0sE", "BMD0sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_bp_dzero_st_lnu * __dss_pi_plus_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

charged_B_to_Dzerostar_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BMD0sE", "BMD0sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_bp_dzero_st_lnu * __dss_pi_zero_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMD0sE", "BMD0sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_bp_dzero_st_lnu * __dss_pi_zero_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Dzerostar_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BZD0sE", "BZD0sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_b0_dzero_st_lnu * __dss_pi_plus_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZD0sE", "BZD0sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_b0_dzero_st_lnu * __dss_pi_plus_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Dzerostar_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BZD0sE", "BZD0sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_b0_dzero_st_lnu * __dss_pi_zero_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZD0sE", "BZD0sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_b0_dzero_st_lnu * __dss_pi_zero_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

# Doneprime Fractions
charged_B_to_Doneprime_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BMDp1E", "BMDp1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_bp_done_prime_lnu * __dss_pi_plus_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMDp1E", "BMDp1Mu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_bp_done_prime_lnu * __dss_pi_plus_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

charged_B_to_Doneprime_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BMDp1E", "BMDp1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_bp_done_prime_lnu * __dss_pi_zero_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMDp1E", "BMDp1Mu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_bp_done_prime_lnu * __dss_pi_zero_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Doneprime_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BZDp1E", "BZDp1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_b0_done_prime_lnu * __dss_pi_plus_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZDp1E", "BZDp1Mu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_b0_done_prime_lnu * __dss_pi_plus_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Doneprime_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BZDp1E", "BZDp1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_b0_done_prime_lnu * __dss_pi_zero_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZDp1E", "BZDp1Mu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_b0_done_prime_lnu * __dss_pi_zero_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

# Doneprime Fractions
charged_B_to_Done_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BMD1E", "BMD1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_bp_done_lnu * __dss_pi_plus_factor * __dss_charged_done_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMD1E", "BMD1Mu"]]),
        fraction=get_one_sided_event_fraction(
            bf=bf_bp_done_lnu * __dss_pi_plus_factor * __dss_charged_done_factor, bf_removed=0
        ),
    ),
)  # type: Tuple[DataTypeFraction, ...]

charged_B_to_Done_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BMD1E", "BMD1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_bp_done_lnu * __dss_pi_zero_factor * __dss_charged_done_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMD1E", "BMD1Mu"]]),
        fraction=get_one_sided_event_fraction(
            bf=bf_bp_done_lnu * __dss_pi_zero_factor * __dss_charged_done_factor, bf_removed=0
        ),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Done_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BZD1E", "BZD1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_b0_done_lnu * __dss_pi_plus_factor * __dss_mixed_done_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZD1E", "BZD1Mu"]]),
        fraction=get_one_sided_event_fraction(
            bf=bf_b0_done_lnu * __dss_pi_plus_factor * __dss_mixed_done_factor, bf_removed=0
        ),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Done_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BZD1E", "BZD1Mu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_b0_done_lnu * __dss_pi_zero_factor * __dss_mixed_done_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZD1E", "BZD1Mu"]]),
        fraction=get_one_sided_event_fraction(
            bf=bf_b0_done_lnu * __dss_pi_zero_factor * __dss_mixed_done_factor, bf_removed=0
        ),
    ),
)  # type: Tuple[DataTypeFraction, ...]

# Dtwostar Fractions
charged_B_to_Dtwostar_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BMD2sE", "BMD2sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_bp_dtwo_st_lnu * __dss_pi_plus_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMD2sE", "BMD2sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_bp_dtwo_st_lnu * __dss_pi_plus_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

charged_B_to_Dtwostar_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BMD2sE", "BMD2sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_bp_dtwo_st_lnu * __dss_pi_zero_factor,
            bf_removed=2 * belle_charged_b_bfs.sum_of_belle_removed_bp_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_charged_data_types if dt not in ["BMD2sE", "BMD2sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_bp_dtwo_st_lnu * __dss_pi_zero_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Dtwostar_l_nu_via_pi_plus_fraction = (
    DataTypeFraction(
        data_types=("BZD2sE", "BZD2sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_plus_factor,
            bf_2=bf_b0_dtwo_st_lnu * __dss_pi_plus_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZD2sE", "BZD2sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_b0_dtwo_st_lnu * __dss_pi_plus_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

neutral_B_to_Dtwostar_l_nu_via_pi_zero_fraction = (
    DataTypeFraction(
        data_types=("BZD2sE", "BZD2sMu"),
        fraction=get_two_sided_event_fraction_with_two_bfs(
            bf_1=1 * __dss_pi_zero_factor,
            bf_2=bf_b0_dtwo_st_lnu * __dss_pi_zero_factor,
            bf_removed=2 * belle_mixed_b_bfs.sum_of_belle_removed_b0_bfs,
        ),
    ),
    DataTypeFraction(
        data_types=tuple([dt for dt in dss_b_neutral_data_types if dt not in ["BZD2sE", "BZD2sMu"]]),
        fraction=get_one_sided_event_fraction(bf=bf_b0_dtwo_st_lnu * __dss_pi_zero_factor, bf_removed=0),
    ),
)  # type: Tuple[DataTypeFraction, ...]

charged_B_charged_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.dss_sig_queries["ChargedBDSl"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["ChargedBDl"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["ChargedBDzeroStarPiP"].id: charged_B_to_Dzerostar_l_nu_via_pi_plus_fraction,
    bf_fit_utils.dss_sig_queries["ChargedBDonePrimePiP"].id: charged_B_to_Doneprime_l_nu_via_pi_plus_fraction,
    bf_fit_utils.dss_sig_queries["ChargedBDonePiP"].id: charged_B_to_Done_l_nu_via_pi_plus_fraction,
    bf_fit_utils.dss_sig_queries["ChargedBDtwoStarPiP"].id: charged_B_to_Dtwostar_l_nu_via_pi_plus_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

neutral_B_charged_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.dss_sig_queries["NeutralBDSl"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["NeutralBDl"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["NeutralBDzeroStarPiP"].id: neutral_B_to_Dzerostar_l_nu_via_pi_plus_fraction,
    bf_fit_utils.dss_sig_queries["NeutralBDonePrimePiP"].id: neutral_B_to_Doneprime_l_nu_via_pi_plus_fraction,
    bf_fit_utils.dss_sig_queries["NeutralBDonePiP"].id: neutral_B_to_Done_l_nu_via_pi_plus_fraction,
    bf_fit_utils.dss_sig_queries["NeutralBDtwoStarPiP"].id: neutral_B_to_Dtwostar_l_nu_via_pi_plus_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

charged_B_neutral_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.dss_sig_queries["ChargedBDSl"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["ChargedBDl"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["ChargedBDzeroStarPiZ"].id: charged_B_to_Dzerostar_l_nu_via_pi_zero_fraction,
    bf_fit_utils.dss_sig_queries["ChargedBDonePrimePiZ"].id: charged_B_to_Doneprime_l_nu_via_pi_zero_fraction,
    bf_fit_utils.dss_sig_queries["ChargedBDonePiZ"].id: charged_B_to_Done_l_nu_via_pi_zero_fraction,
    bf_fit_utils.dss_sig_queries["ChargedBDtwoStarPiZ"].id: charged_B_to_Dtwostar_l_nu_via_pi_zero_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

neutral_B_neutral_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.dss_sig_queries["NeutralBDSl"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["NeutralBDl"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["NeutralBDzeroStarPiZ"].id: neutral_B_to_Dzerostar_l_nu_via_pi_zero_fraction,
    bf_fit_utils.dss_sig_queries["NeutralBDonePrimePiZ"].id: neutral_B_to_Doneprime_l_nu_via_pi_zero_fraction,
    bf_fit_utils.dss_sig_queries["NeutralBDonePiZ"].id: neutral_B_to_Done_l_nu_via_pi_zero_fraction,
    bf_fit_utils.dss_sig_queries["NeutralBDtwoStarPiZ"].id: neutral_B_to_Dtwostar_l_nu_via_pi_zero_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

decoupled_charged_B_charged_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedDs_DZ"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedD_DZ"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedDs_DSZ"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedD_DSZ"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries[
        "ChargedBDzeroStarPiP"
    ].id: charged_B_to_Dzerostar_l_nu_via_pi_plus_fraction,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries[
        "ChargedBDonePrimePiP"
    ].id: charged_B_to_Doneprime_l_nu_via_pi_plus_fraction,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries[
        "ChargedBDonePiP"
    ].id: charged_B_to_Done_l_nu_via_pi_plus_fraction,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries[
        "ChargedBDtwoStarPiP"
    ].id: charged_B_to_Dtwostar_l_nu_via_pi_plus_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

decoupled_neutral_B_charged_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroDs_DM"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroD_DM"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroDs_DSM"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroD_DSM"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries[
        "NeutralBDzeroStarPiP"
    ].id: neutral_B_to_Dzerostar_l_nu_via_pi_plus_fraction,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries[
        "NeutralBDonePrimePiP"
    ].id: neutral_B_to_Doneprime_l_nu_via_pi_plus_fraction,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries[
        "NeutralBDonePiP"
    ].id: neutral_B_to_Done_l_nu_via_pi_plus_fraction,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries[
        "NeutralBDtwoStarPiP"
    ].id: neutral_B_to_Dtwostar_l_nu_via_pi_plus_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

decoupled_charged_B_neutral_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries["NormBchargedDs_DZ"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries["NormBchargedD_DZ"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries["NormBchargedDs_DSZ"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries["NormBchargedD_DSZ"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries[
        "ChargedBDzeroStarPiZ"
    ].id: charged_B_to_Dzerostar_l_nu_via_pi_zero_fraction,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries[
        "ChargedBDonePrimePiZ"
    ].id: charged_B_to_Doneprime_l_nu_via_pi_zero_fraction,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries[
        "ChargedBDonePiZ"
    ].id: charged_B_to_Done_l_nu_via_pi_zero_fraction,
    bf_fit_utils.decoupled_charged_B_neutral_pi_sig_queries[
        "ChargedBDtwoStarPiZ"
    ].id: charged_B_to_Dtwostar_l_nu_via_pi_zero_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

decoupled_neutral_B_neutral_pi_sig_dss_data_type_mapping = {
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries["NormBzeroDs_DM"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries["NormBzeroD_DM"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries["NormBzeroDs_DSM"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries["NormBzeroD_DSM"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries[
        "NeutralBDzeroStarPiZ"
    ].id: neutral_B_to_Dzerostar_l_nu_via_pi_zero_fraction,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries[
        "NeutralBDonePrimePiZ"
    ].id: neutral_B_to_Doneprime_l_nu_via_pi_zero_fraction,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries[
        "NeutralBDonePiZ"
    ].id: neutral_B_to_Done_l_nu_via_pi_zero_fraction,
    bf_fit_utils.decoupled_neutral_B_neutral_pi_sig_queries[
        "NeutralBDtwoStarPiZ"
    ].id: neutral_B_to_Dtwostar_l_nu_via_pi_zero_fraction,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]


# Normalization Fit Info
normalization_fit_coupled_charged_B_sig_dss_data_type_mapping = {
    bf_fit_utils.dss_sig_queries["ChargedBDSl"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["ChargedBDl"].id: charged_B_to_D_l_nu_fractions,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

normalization_fit_coupled_neutral_B_sig_dss_data_type_mapping = {
    bf_fit_utils.dss_sig_queries["NeutralBDSl"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.dss_sig_queries["NeutralBDl"].id: neutral_B_to_D_l_nu_fractions,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

normalization_fit_decoupled_charged_B_sig_dss_data_type_mapping = {
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedDs_DZ"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedD_DZ"].id: charged_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedDs_DSZ"].id: charged_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_charged_B_charged_pi_sig_queries["NormBchargedD_DSZ"].id: charged_B_to_D_l_nu_fractions,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]

normalization_fit_decoupled_neutral_B_sig_dss_data_type_mapping = {
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroDs_DM"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroD_DM"].id: neutral_B_to_D_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroDs_DSM"].id: neutral_B_to_D_st_l_nu_fractions,
    bf_fit_utils.decoupled_neutral_B_charged_pi_sig_queries["NormBzeroD_DSM"].id: neutral_B_to_D_l_nu_fractions,
}  # type: Dict[int, Tuple[DataTypeFraction, ...]]
