"""
Handling of systematics for the new dedicated fit.

Felix Metzner 2023
"""

from dataclasses import dataclass
from collections.abc import Mapping
from typing import Tuple, Dict, Type, List, Any, ClassVar, Iterator, Optional, Generator

from rdstar.utilities.logging import log_to_default

from rdstar.offline_analysis.fitting.dedicated_fit_approach import rdstar_systematics as rdstar_sys
from rdstar.offline_analysis.fitting.dedicated_fit_approach.rdstar_systematics import (
    BKGHandling,
    AbstractSysInfo,
    AbstractBKGHandlingInfo,
)


__all__ = [
    "BKGHandlingInfo",
    "BKGHandlingInfoContainer",
    "SystematicsInfo",
    "RDStarSystematicsInfos",
    "RDStarSystematicsContainer",
]


logger = log_to_default(name=__name__)


@dataclass(frozen=True)
class BKGHandlingInfo:
    status: BKGHandling
    info_class: Type[AbstractBKGHandlingInfo]
    first_bkg_yield_param_id: int

    @property
    def bkg_constraint_key(self) -> str:
        return self.info_class.get_systematics_key()

    @property
    def bkg_yield_ids(self) -> Tuple[int, ...]:
        return self.info_class.get_bkg_yield_indices()

    @property
    def bkg_constraint_id_offsets(self) -> Tuple[int, ...]:
        return tuple(i - self.first_bkg_yield_param_id for i in self.info_class.get_bkg_yield_indices())

    @property
    def bkg_id_char(self) -> str:
        return self.info_class.get_bkg_id_char()

    @property
    def bkg_status_code(self) -> int:
        return BKGHandling.bkg_handling_to_int_code(item=self.status)

    @property
    def info_string(self) -> str:
        return f"{self.bkg_id_char}{self.bkg_status_code}"


class BKGHandlingInfoContainer(Mapping):
    __bkg_handling_infos: ClassVar[Tuple[Type[AbstractBKGHandlingInfo], ...]] = (
        rdstar_sys.BKGHandlingDss,
        rdstar_sys.BKGHandlingBBBar,
        rdstar_sys.BKGHandlingContinuum,
    )

    _bkg_index_start: ClassVar[int] = 4
    _bkg_index_end: ClassVar[int] = 9

    def __init__(
        self,
        dss_bkg: BKGHandling,
        bbbar_bkg: BKGHandling,
        continuum_bkg: BKGHandling,
        bkg_constraint_param_id_start: int,
    ) -> None:
        self._check_base_bkg_handling_infos()

        self.bkg_constraint_param_id_start: int = bkg_constraint_param_id_start

        self._bkg_handling_infos_mapping: Dict[str, Type[AbstractBKGHandlingInfo]] = {
            c.get_systematics_key(): c for c in self.__bkg_handling_infos
        }

        self.check_bkg_handling_status(bkg_handling=dss_bkg, info_text="dss_bkg")
        self.check_bkg_handling_status(bkg_handling=bbbar_bkg, info_text="bbbar_bkg")
        self.check_bkg_handling_status(bkg_handling=continuum_bkg, info_text="continuum_bkg")

        _first_bkg_yield_param_id: int = min(v for i in self.__bkg_handling_infos for v in i.get_bkg_yield_indices())

        # fmt: off
        self._bkg_handling_status_infos: Tuple[BKGHandlingInfo, ...] = (
            BKGHandlingInfo(status=dss_bkg, info_class=rdstar_sys.BKGHandlingDss, first_bkg_yield_param_id=_first_bkg_yield_param_id),
            BKGHandlingInfo(status=bbbar_bkg, info_class=rdstar_sys.BKGHandlingBBBar, first_bkg_yield_param_id=_first_bkg_yield_param_id),
            BKGHandlingInfo(status=continuum_bkg, info_class=rdstar_sys.BKGHandlingContinuum, first_bkg_yield_param_id=_first_bkg_yield_param_id),
        )
        # fmt: on

        for bkg_h_status_info in self._bkg_handling_status_infos:
            assert all(i >= 0.0 for i in bkg_h_status_info.bkg_constraint_id_offsets), (
                bkg_h_status_info.bkg_constraint_key,
                [i for i in bkg_h_status_info.bkg_constraint_id_offsets if i < 0.0],
            )

        self._bkg_handling_status_mapping: Dict[str, BKGHandlingInfo] = {
            i.info_class.get_systematics_key(): i for i in self._bkg_handling_status_infos
        }

        assert len(self._bkg_handling_status_mapping) == len(self.__bkg_handling_infos), (
            len(self._bkg_handling_status_mapping),
            len(self.__bkg_handling_infos),
        )

        for bkg_key, bkg_handling in self._bkg_handling_status_mapping.items():
            assert bkg_handling.status in BKGHandling, (bkg_key, bkg_handling.status, bkg_handling)

    @staticmethod
    def is_valid_bkg_handling_status(bkg_handling: BKGHandling) -> bool:
        return bkg_handling in BKGHandling

    def check_bkg_handling_status(self, bkg_handling: Any, info_text: Optional[str] = None) -> None:
        assert isinstance(bkg_handling, BKGHandling) and self.is_valid_bkg_handling_status(bkg_handling=bkg_handling), (
            bkg_handling.name,
            bkg_handling.value,
            BKGHandling.get_valid_members_info(),
            info_text,
        )

    @property
    def values_as_info_str(self) -> str:
        return "BKG" + "".join([i.info_string for i in self._bkg_handling_status_infos])

    def _check_base_bkg_handling_infos(self) -> None:
        for bkg_constrain_info in self.__bkg_handling_infos:
            assert len(bkg_constrain_info.get_bkg_yield_indices()) == bkg_constrain_info.number_of_nuisances(), (
                bkg_constrain_info.get_systematics_key(),
                bkg_constrain_info.get_bkg_yield_indices(),
                len(bkg_constrain_info.get_bkg_yield_indices()),
                bkg_constrain_info.number_of_nuisances(),
            )

            assert all(v >= self._bkg_index_start for v in bkg_constrain_info.get_bkg_yield_indices()), (
                bkg_constrain_info.get_systematics_key(),
                [v for v in bkg_constrain_info.get_bkg_yield_indices() if v < self._bkg_index_start],
            )
            assert all(v <= self._bkg_index_end for v in bkg_constrain_info.get_bkg_yield_indices()), (
                bkg_constrain_info.get_systematics_key(),
                [v for v in bkg_constrain_info.get_bkg_yield_indices() if v > self._bkg_index_end],
            )

        all_keys: Tuple[str, ...] = tuple(i.get_systematics_key() for i in self.__bkg_handling_infos)
        all_bkg_yield_ids: Tuple[int, ...] = tuple(
            v for i in self.__bkg_handling_infos for v in i.get_bkg_yield_indices()
        )
        all_bkg_id_chars: Tuple[str, ...] = tuple(i.get_bkg_id_char() for i in self.__bkg_handling_infos)

        key_duplicates: List[str] = list(set([v for v in all_keys if all_keys.count(v) > 1]))
        assert len(key_duplicates) == 0, key_duplicates
        index_duplicates: List[int] = list(set([v for v in all_bkg_yield_ids if all_bkg_yield_ids.count(v) > 1]))
        assert len(index_duplicates) == 0, index_duplicates
        id_char_duplicates: List[str] = list(set([v for v in all_bkg_id_chars if all_bkg_id_chars.count(v) > 1]))
        assert len(id_char_duplicates) == 0, id_char_duplicates

    def __getitem__(self, item: str) -> BKGHandlingInfo:
        return self._bkg_handling_status_mapping[item]

    def __iter__(self) -> Iterator[str]:
        return iter(self._bkg_handling_status_mapping)

    def __len__(self) -> int:
        return len(self._bkg_handling_status_mapping)

    @property
    def number_of_bkg_constraints(self) -> int:
        return sum([v.number_of_nuisances() for v in self.__bkg_handling_infos])

    @property
    def bkg_constraint_nuisance_param_ids(self) -> Tuple[int, ...]:
        return tuple(
            self.bkg_constraint_param_id_start + i
            for v in self._bkg_handling_status_infos
            for i in v.bkg_constraint_id_offsets
        )

    def get_modified_bkg_handlers(
        self,
        dss_bkg: Optional[BKGHandling] = None,
        bbbar_bkg: Optional[BKGHandling] = None,
        continuum_bkg: Optional[BKGHandling] = None,
    ) -> "BKGHandlingInfoContainer":

        old_dss_bkg_status: BKGHandling = self[rdstar_sys.BKGHandlingDss.get_systematics_key()].status
        old_bbbar_bkg_status: BKGHandling = self[rdstar_sys.BKGHandlingBBBar.get_systematics_key()].status
        old_continuum_bkg_status: BKGHandling = self[rdstar_sys.BKGHandlingContinuum.get_systematics_key()].status

        dss_bkg_status: BKGHandling = old_dss_bkg_status if dss_bkg is None else dss_bkg
        bbbar_bkg_status: BKGHandling = old_bbbar_bkg_status if bbbar_bkg is None else bbbar_bkg
        continuum_bkg_status: BKGHandling = old_continuum_bkg_status if continuum_bkg is None else continuum_bkg

        new_container: BKGHandlingInfoContainer = BKGHandlingInfoContainer(
            dss_bkg=dss_bkg_status,
            bbbar_bkg=bbbar_bkg_status,
            continuum_bkg=continuum_bkg_status,
            bkg_constraint_param_id_start=self.bkg_constraint_param_id_start,
        )

        assert new_container.values_as_info_str != self.values_as_info_str, (
            (old_dss_bkg_status.value, dss_bkg if dss_bkg is None else dss_bkg.value),
            (old_bbbar_bkg_status.value, bbbar_bkg if bbbar_bkg is None else bbbar_bkg.value),
            (old_continuum_bkg_status.value, continuum_bkg if continuum_bkg is None else continuum_bkg.value),
            self.values_as_info_str,
            new_container.values_as_info_str,
        )

        return new_container

    def get_bkg_handler_with_dss_bkg_fixed(self) -> "BKGHandlingInfoContainer":
        assert self[rdstar_sys.BKGHandlingDss.get_systematics_key()].status is not BKGHandling.BKG_FIXED
        return self.get_modified_bkg_handlers(dss_bkg=BKGHandling.BKG_FIXED)

    def get_bkg_handler_with_bbbar_bkg_fixed(self) -> "BKGHandlingInfoContainer":
        assert self[rdstar_sys.BKGHandlingBBBar.get_systematics_key()].status is not BKGHandling.BKG_FIXED
        return self.get_modified_bkg_handlers(bbbar_bkg=BKGHandling.BKG_FIXED)

    def get_bkg_handler_with_continuum_bkg_fixed(self) -> "BKGHandlingInfoContainer":
        assert self[rdstar_sys.BKGHandlingContinuum.get_systematics_key()].status is not BKGHandling.BKG_FIXED
        return self.get_modified_bkg_handlers(continuum_bkg=BKGHandling.BKG_FIXED)

    def get_bkg_handler_infos_for_evaluation(self) -> Generator[Tuple[str, "BKGHandlingInfoContainer"], None, None]:
        for bkg_name in self._bkg_handling_status_mapping:
            if self[bkg_name].status is BKGHandling.BKG_FIXED:
                continue
            elif bkg_name == rdstar_sys.BKGHandlingDss.get_systematics_key():
                yield bkg_name, self.get_bkg_handler_with_dss_bkg_fixed()
            elif bkg_name == rdstar_sys.BKGHandlingBBBar.get_systematics_key():
                yield bkg_name, self.get_bkg_handler_with_bbbar_bkg_fixed()
            elif bkg_name == rdstar_sys.BKGHandlingContinuum.get_systematics_key():
                yield bkg_name, self.get_bkg_handler_with_continuum_bkg_fixed()
            else:
                raise KeyError(f"Unknown background constraint key '{bkg_name}'!")


@dataclass(frozen=True)
class SystematicsInfo:
    is_active: bool
    info_class: Type[AbstractSysInfo]
    first_nuisance_param_id: int

    @property
    def systematics_key(self) -> str:
        return self.info_class.get_systematics_key()

    @property
    def latex_str(self) -> str:
        return self.info_class.get_latex_str()

    @property
    def number_of_nuisances(self) -> int:
        return self.info_class.number_of_nuisances()

    @property
    def nuisance_param_ids(self) -> Tuple[int, ...]:
        return tuple(i + self.first_nuisance_param_id for i in range(self.number_of_nuisances))

    @property
    def is_additive_sys(self) -> bool:
        return self.info_class.get_systematics_type() == "additive"

    @property
    def is_multiplicative_sys(self) -> bool:
        return self.info_class.get_systematics_type() == "multiplicative"

    @property
    def nuisance_param_id(self) -> int:
        assert self.number_of_nuisances == 1, (self.systematics_key, self.number_of_nuisances)
        return self.first_nuisance_param_id

    @property
    def always_fixed_param_ids(self) -> Optional[Tuple[int, ...]]:
        fixed_params: Optional[Tuple[int, ...]] = self.info_class.get_always_fixed_parameter_indices()
        if fixed_params is None:
            return None
        assert len(fixed_params) <= self.number_of_nuisances, (
            self.systematics_key,
            len(fixed_params),
            self.number_of_nuisances,
            fixed_params,
        )
        assert all(0 <= v < self.number_of_nuisances for v in fixed_params), (
            self.systematics_key,
            fixed_params,
            self.number_of_nuisances,
        )

        return tuple(i + self.first_nuisance_param_id for i in fixed_params)


class RDStarSystematicsInfos:
    ordered_systematics_infos: ClassVar[Tuple[Type[AbstractSysInfo], ...]] = (
        # Multiplicative Systematics:
        rdstar_sys.MultSysMCStatistics,
        # rdstar_sys.MultSysBFTauToLNuNu,  # TODO: Implement and use!
        # rdstar_sys.MultSysDwwosMesonBF,  # TODO: Implement and use!
        # rdstar_sys.MultSysFFSignal,  # TODO: Implement and use!
        # rdstar_sys.MultSysFFNorm,  # TODO: Implement and use!
        rdstar_sys.MultSysQEDModel,
        rdstar_sys.MultSysTauZeroPlusRatio,
        # rdstar_sys.MultSysPIDLepton,  # TODO: Implement and use!
        # rdstar_sys.MultSysPIDHadron,  # TODO: Implement and use!
        # rdstar_sys.MultSysSlowPion,  # TODO: Implement and use!
        # rdstar_sys.MultSysTracking,  # TODO: Implement and use!
        rdstar_sys.MultSysFEIEff,
        # rdstar_sys.MultSysFEISlowPion,  # TODO: Implement and use!
        # rdstar_sys.MultSysKShortEff,  # TODO: Implement and use!
        # rdstar_sys.MultSysPiZeroEff,  # TODO: Implement and use!
        #
        # Additive Systematics:
        rdstar_sys.AddSysMCStatistics,
        # rdstar_sys.AddSysDwwosMesonBF,  # TODO: Implement and use!
        # rdstar_sys.AddSysBFBtoDwwos,  # TODO: Implement and use!
        rdstar_sys.AddSysFFSignal,
        rdstar_sys.AddSysFFNorm,
        rdstar_sys.AddSysQEDModel,
        # rdstar_sys.AddSysDSSBF,  # TODO: Implement and use!
        rdstar_sys.AddSysDSSFFWide,
        rdstar_sys.AddSysDSSFFNarrow,
        # rdstar_sys.AddSysGapBKGModelling,  # TODO: Implement and use!
        # rdstar_sys.AddSysGapBKGBFComposition,  # TODO: Implement and use!
        # rdstar_sys.AddSysBBbarBKGBFComposition,  # TODO: Implement and use!
        # rdstar_sys.AddSysContinuumBKGComposition,  # TODO: Implement and use!
        rdstar_sys.AddSysPIDLepton,
        rdstar_sys.AddSysPIDLeptonFake,
        rdstar_sys.AddSysPIDHadron,
        rdstar_sys.AddSysSlowPion,
        rdstar_sys.AddSysTracking,
        # rdstar_sys.AddSysFEISlowPion,  # TODO: Implement and use!
        rdstar_sys.AddSysKShortEff,
        # rdstar_sys.AddSysPiZeroEff,  # TODO: Implement and use!
    )

    all_sys_info_names: ClassVar[Tuple[str, ...]] = tuple(v.get_systematics_key() for v in ordered_systematics_infos)

    total_number_of_nuisance_parameters: ClassVar[int] = sum(i.number_of_nuisances() for i in ordered_systematics_infos)

    @classmethod
    def run_checks(cls) -> None:
        assert len(cls.all_sys_info_names) == len(set(cls.all_sys_info_names)), (
            [n for n in cls.all_sys_info_names if cls.all_sys_info_names.count(n) > 1],
        )


class RDStarSystematicsContainer(Mapping):

    empty_systematics_container_setup_name: str = "stats_only_sys_setup"
    full_systematics_container_setup_name: str = "full_sys_setup"

    systematics_scan_container_setup_name: str = "sys_scan_setup"

    def __init__(
        self,
        name: str,
        first_sys_nuisance_param_index: int,
        inactive_sys_names: Optional[Tuple[str, ...]],
    ) -> None:
        self.name: str = name

        assert first_sys_nuisance_param_index >= 0, (self.name, first_sys_nuisance_param_index)
        self.first_sys_nuisance_param_index: int = first_sys_nuisance_param_index

        self.inactive_sys_names: Tuple[str, ...] = tuple() if inactive_sys_names is None else inactive_sys_names
        assert all(n in RDStarSystematicsInfos.all_sys_info_names for n in self.inactive_sys_names), (
            [n for n in self.inactive_sys_names if n not in RDStarSystematicsInfos.all_sys_info_names],
        )

        _systematics_infos: List[SystematicsInfo] = []
        current_first_free_nuisance_param_index: int = self.first_sys_nuisance_param_index
        for sys_info in RDStarSystematicsInfos.ordered_systematics_infos:
            _systematics_infos.append(
                SystematicsInfo(
                    is_active=sys_info.get_systematics_key() not in self.inactive_sys_names,
                    info_class=sys_info,
                    first_nuisance_param_id=current_first_free_nuisance_param_index,
                )
            )
            current_first_free_nuisance_param_index += sys_info.number_of_nuisances()

        self._sys_infos: Tuple[SystematicsInfo, ...] = tuple(_systematics_infos)
        self._sys_info_mapping: Dict[str, SystematicsInfo] = {v.systematics_key: v for v in self._sys_infos}

        self._any_additive_systematic_is_active: Optional[bool] = None

        self.run_checks()

    def __getitem__(self, item: str) -> SystematicsInfo:
        return self._sys_info_mapping[item]

    def __iter__(self) -> Iterator[str]:
        return iter(self._sys_info_mapping)

    def __len__(self) -> int:
        return len(self._sys_infos)

    @property
    def has_additive_systematics(self) -> bool:
        if self._any_additive_systematic_is_active is None:
            self._any_additive_systematic_is_active = self._get_is_any_add_sys_active()
        return self._any_additive_systematic_is_active

    def _get_is_any_add_sys_active(self) -> bool:
        return any(s.is_active and s.is_additive_sys for s in self._sys_infos)

    def set_states(self, states_infos: Dict[str, bool]) -> None:
        assert all(k in self.systematics_names for k in states_infos), (
            [k for k in states_infos if k not in self.systematics_names],
        )

        current_states: Dict[str, bool] = self.get_states()
        for sys_name, sys_state in states_infos.items():
            if sys_state != current_states[sys_name]:
                self._reset_sys_info_state(sys_name=sys_name, new_state=sys_state)

    def get_states(self) -> Dict[str, bool]:
        return {i.systematics_key: i.is_active for i in self._sys_infos}

    def _reset_sys_info_state(self, sys_name: str, new_state: bool) -> None:
        assert sys_name in self._sys_info_mapping, (sys_name,)

        old_sys_info: SystematicsInfo = self[sys_name]
        assert new_state != old_sys_info.is_active, (sys_name, new_state, old_sys_info.is_active)

        new_sys_info: SystematicsInfo = SystematicsInfo(
            is_active=new_state,
            info_class=old_sys_info.info_class,
            first_nuisance_param_id=old_sys_info.first_nuisance_param_id,
        )

        self._sys_info_mapping[sys_name] = new_sys_info
        new_sys_info_tuple: Tuple[SystematicsInfo, ...] = tuple(
            new_sys_info if sys_name == v.systematics_key else v for v in self._sys_infos
        )
        self._sys_infos = new_sys_info_tuple
        self._any_additive_systematic_is_active = None

    @property
    def always_fixed_nuisance_parameter_indices(self) -> Tuple[int, ...]:
        return tuple(
            v for i in self._sys_infos for v in ([] if i.always_fixed_param_ids is None else i.always_fixed_param_ids)
        )

    @property
    def ordered_systematics_names(self) -> Tuple[str, ...]:
        return tuple(v.get_systematics_key() for v in RDStarSystematicsInfos.ordered_systematics_infos)

    @property
    def systematics_names(self) -> Tuple[str, ...]:
        return tuple(self.keys())

    @property
    def all_nuisance_parameter_ids(self) -> Tuple[int, ...]:
        return tuple(i for v in self._sys_infos for i in v.nuisance_param_ids)

    @property
    def number_of_nuisance_parameters(self) -> int:
        return len(self.all_nuisance_parameter_ids)

    def encode_systematics_state_to_hex(self) -> str:
        binary_str: str = "".join([str(int(i.is_active)) for i in self._sys_infos])
        return hex(int(binary_str, 2))

    @staticmethod
    def decode_hex_systematics_state(state_hex_str: str) -> Tuple[bool, ...]:
        binary_str: str = str(bin(int(state_hex_str, 16)))
        return tuple(bool(int(v)) for v in binary_str[2:])

    @property
    def id_hash(self) -> str:
        return str(self.encode_systematics_state_to_hex())

    @staticmethod
    def get_active_sys_names_map_from_hash(state_hex_str: str) -> Dict[str, bool]:
        states: Tuple[bool, ...] = RDStarSystematicsContainer.decode_hex_systematics_state(state_hex_str=state_hex_str)
        return {n: s for n, s in zip(RDStarSystematicsInfos.all_sys_info_names, states)}

    def run_checks(self) -> None:
        param_ids: Tuple[int, ...] = self.all_nuisance_parameter_ids

        assert RDStarSystematicsInfos.total_number_of_nuisance_parameters == self.number_of_nuisance_parameters, (
            RDStarSystematicsInfos.total_number_of_nuisance_parameters,
            self.number_of_nuisance_parameters,
        )

        assert len(param_ids) == len(set(param_ids)), ([n for n in param_ids if param_ids.count(n) > 1],)

    def get_inactive_systematics_setup(self) -> "RDStarSystematicsContainer":
        return RDStarSystematicsContainer(
            name=self.empty_systematics_container_setup_name,
            first_sys_nuisance_param_index=self.first_sys_nuisance_param_index,
            inactive_sys_names=RDStarSystematicsInfos.all_sys_info_names,
        )

    def get_sys_setup_without(self, excluded_sys_name: str) -> "RDStarSystematicsContainer":
        assert excluded_sys_name in RDStarSystematicsInfos.all_sys_info_names, excluded_sys_name
        if not len(self.inactive_sys_names) == 0:
            raise RuntimeError(
                "RDStarSystematicsContainer with a single excluded systematic "
                "should be based on the full systematics setup! "
                "However, the following systematics are already excluded:\n\t-" + "\n\t-".join(self.inactive_sys_names)
            )
        return RDStarSystematicsContainer(
            name=f"{self.systematics_scan_container_setup_name}_wo_{excluded_sys_name}",
            first_sys_nuisance_param_index=self.first_sys_nuisance_param_index,
            inactive_sys_names=(excluded_sys_name,),
        )

    def get_sys_setup_infos_for_evaluation(self) -> Generator[Tuple[str, "RDStarSystematicsContainer"], None, None]:
        for sys_name in RDStarSystematicsInfos.all_sys_info_names:
            yield sys_name, self.get_sys_setup_without(excluded_sys_name=sys_name)

    @property
    def mult_sys__mc_statistics(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysMCStatistics.get_systematics_key()]

    @property
    def mult_sys__bf_tau_to_lnunu(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysBFTauToLNuNu.get_systematics_key()]

    @property
    def mult_sys__dwwos_meson_bf(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysDwwosMesonBF.get_systematics_key()]

    @property
    def mult_sys__ff_signal(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysFFSignal.get_systematics_key()]

    @property
    def mult_sys__ff_norm(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysFFNorm.get_systematics_key()]

    @property
    def mult_sys__qed_model(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysQEDModel.get_systematics_key()]

    @property
    def mult_sys__tau_zp_ratio(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysTauZeroPlusRatio.get_systematics_key()]

    @property
    def mult_sys__pid_lepton(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysPIDLepton.get_systematics_key()]

    @property
    def mult_sys__pid_hadron(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysPIDHadron.get_systematics_key()]

    @property
    def mult_sys__slow_pion(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysSlowPion.get_systematics_key()]

    @property
    def mult_sys__tracking(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysTracking.get_systematics_key()]

    @property
    def mult_sys__fei_eff(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysFEIEff.get_systematics_key()]

    @property
    def mult_sys__fei_slow_pion(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysFEISlowPion.get_systematics_key()]

    @property
    def mult_sys__k_short_eff(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysKShortEff.get_systematics_key()]

    @property
    def mult_sys__pi_zero_eff(self) -> SystematicsInfo:
        return self[rdstar_sys.MultSysPiZeroEff.get_systematics_key()]

    #
    # Additive Systematics:
    @property
    def add_sys__mc_statistics(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysMCStatistics.get_systematics_key()]

    @property
    def add_sys__dwwos_meson_bf(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysDwwosMesonBF.get_systematics_key()]

    @property
    def add_sys__bf_b_to_dwwos(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysBFBtoDwwos.get_systematics_key()]

    @property
    def add_sys__ff_signal(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysFFSignal.get_systematics_key()]

    @property
    def add_sys__ff_norm(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysFFNorm.get_systematics_key()]

    @property
    def add_sys__qed_model(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysQEDModel.get_systematics_key()]

    @property
    def add_sys__dss_bf(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysDSSBF.get_systematics_key()]

    @property
    def add_sys__dss_wide_ff(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysDSSFFWide.get_systematics_key()]

    @property
    def add_sys__dss_narrow_ff(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysDSSFFNarrow.get_systematics_key()]

    @property
    def add_sys__gap_bkg_modelling(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysGapBKGModelling.get_systematics_key()]

    @property
    def add_sys__gap_bkg_bf_composition(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysGapBKGBFComposition.get_systematics_key()]

    @property
    def add_sys__bbbar_bkg_bf_composition(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysBBbarBKGBFComposition.get_systematics_key()]

    @property
    def add_sys__continuum_bkg_composition(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysContinuumBKGComposition.get_systematics_key()]

    @property
    def add_sys__pid_lepton(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysPIDLepton.get_systematics_key()]

    @property
    def add_sys__pid_lepton_fake(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysPIDLeptonFake.get_systematics_key()]

    @property
    def add_sys__pid_hadron(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysPIDHadron.get_systematics_key()]

    @property
    def add_sys__slow_pion(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysSlowPion.get_systematics_key()]

    @property
    def add_sys__tracking(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysTracking.get_systematics_key()]

    @property
    def add_sys__fei_slow_pion(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysFEISlowPion.get_systematics_key()]

    @property
    def add_sys__k_short_eff(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysKShortEff.get_systematics_key()]

    @property
    def add_sys__pi_zero_eff(self) -> SystematicsInfo:
        return self[rdstar_sys.AddSysPiZeroEff.get_systematics_key()]


if __name__ == "__main__":
    RDStarSystematicsInfos.run_checks()

    sys_container: RDStarSystematicsContainer = RDStarSystematicsContainer(
        name="test",
        first_sys_nuisance_param_index=0,
        inactive_sys_names=tuple(),
    )

    assert len(sys_container) == len(RDStarSystematicsInfos.ordered_systematics_infos), (
        len(sys_container),
        len(RDStarSystematicsInfos.ordered_systematics_infos),
    )

    for sys_key, sys_info_value in sys_container.items():
        if sys_info_value.number_of_nuisances == 0:
            print(f"\t {sys_key}: {sys_info_value.number_of_nuisances}")
        else:
            assert sys_info_value.number_of_nuisances == len(sys_info_value.nuisance_param_ids), (
                sys_key,
                sys_info_value.number_of_nuisances,
                len(sys_info_value.nuisance_param_ids),
                sys_info_value.nuisance_param_ids,
            )
