"""
Fit setup info for Continuum calibration fit.

Felix Metzner 2023
"""

from typing import Tuple
from uncertainties import ufloat

from rdstar.utilities.logging import log_to_default

from rdstar.latex_labels.latex_process_labels import LatexDecayProcesses

from rdstar.offline_analysis.reco_modes.reco_mode_infos import TagRecoModeInfos
from rdstar.offline_analysis.mc_matching.old_mc_matching_info import OldMCMatchingIDsQueryInfo

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups.fit_setup_utilities import (
    FitSetupTriplet,
    base_asimov_data_setup_info,
    cont_cal_b_charge_fit_obs_setup,
    base_cont_cal_weight_info,
    cont_cal_bbbar_bkg_sig_ids,
    fei_tag_b_reco_based_fit_component_id_column_name,
    add_fei_tag_b_reco_based_fit_component_id,
    get_sig_ids_with_tag_b_reco_info,
)
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_info_container import (
    FitSetupInfoContainer,
    ComponentInfo,
    TagEfficiencyCorrectionInfo,
    RecoChannelInfo,
)


__all__ = [
    "cont_cal_fit_setup",
]


logger = log_to_default(name=__name__)


# Definition of components:

# (Main) Continuum Components

component_cb_tag_reco_continuum_ids: Tuple[int, ...] = get_sig_ids_with_tag_b_reco_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    fei_reco_type=None,
    fei_reco_charge_type="charged",
)
component_continuum_cb_tag: ComponentInfo = ComponentInfo(
    name="continuum_in_cb_tag",
    comp_index=10,
    latex_label=LatexDecayProcesses.to_continuum_txt,
    sig_ids=component_cb_tag_reco_continuum_ids,
)


component_nb_tag_reco_continuum_ids: Tuple[int, ...] = get_sig_ids_with_tag_b_reco_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    fei_reco_type=None,
    fei_reco_charge_type="neutral",
)
component_continuum_nb_tag: ComponentInfo = ComponentInfo(
    name="continuum_in_nb_tag",
    comp_index=11,
    latex_label=LatexDecayProcesses.to_continuum_txt,
    sig_ids=component_nb_tag_reco_continuum_ids,
)


# Background Components

b_bkg_sig_ids: Tuple[int, ...] = (
    OldMCMatchingIDsQueryInfo.Dl.id,
    OldMCMatchingIDsQueryInfo.DSl.id,
) + cont_cal_bbbar_bkg_sig_ids

# BKG in Charged B Tag

cont_cal_cb_tag_b_bkg_ids: Tuple[int, ...] = get_sig_ids_with_tag_b_reco_info(
    sig_ids=b_bkg_sig_ids,
    fei_reco_type=None,
    fei_reco_charge_type="charged",
)

component_cb_tag_b_bkg: ComponentInfo = ComponentInfo(
    name="cb_tag_b_BKG",
    comp_index=20,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg,
    sig_ids=cont_cal_cb_tag_b_bkg_ids,
)


# BKG in Neutral B Tag

cont_cal_nb_tag_b_bkg_ids: Tuple[int, ...] = get_sig_ids_with_tag_b_reco_info(
    sig_ids=b_bkg_sig_ids,
    fei_reco_type=None,
    fei_reco_charge_type="neutral",
)

component_nb_tag_b_bkg: ComponentInfo = ComponentInfo(
    name="nb_tag_b_BKG",
    comp_index=21,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg,
    sig_ids=cont_cal_nb_tag_b_bkg_ids,
)


cont_cal_fit_comps: Tuple[ComponentInfo, ...] = (
    # (Main) Continuum Components
    component_continuum_cb_tag,
    component_continuum_nb_tag,
    # (Bkg) BBbar Components
    component_cb_tag_b_bkg,
    component_nb_tag_b_bkg,
)

cont_cal_fit_chs: Tuple[RecoChannelInfo, ...] = tuple(
    [
        RecoChannelInfo(
            name="neutralTagB",
            ch_index=0,
            reco_mode_info=TagRecoModeInfos.neutral_tag_reco_modes,
            components=cont_cal_fit_comps,
            observable_infos=cont_cal_b_charge_fit_obs_setup.cont_obs_info_neutral_b,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
        RecoChannelInfo(
            name="chargedTagB",
            ch_index=1,
            reco_mode_info=TagRecoModeInfos.charged_tag_reco_modes,
            components=cont_cal_fit_comps,
            observable_infos=cont_cal_b_charge_fit_obs_setup.cont_obs_info_charged_b,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
    ]
)


cont_cal_fit_tag_eff_corr_info: TagEfficiencyCorrectionInfo = TagEfficiencyCorrectionInfo(
    items=[],
    key_order=(),
)


_cont_cal_asimov_fit_setup = FitSetupInfoContainer(
    components=cont_cal_fit_comps,
    reco_channels=cont_cal_fit_chs,
    observable_infos=cont_cal_b_charge_fit_obs_setup.cont_cal_obs_infos,
    weight_info=base_cont_cal_weight_info,
    data_setup_info=base_asimov_data_setup_info,
    remove_empty_bins=True,
    use_alternative_gap=True,
    tag_efficiency_correction_info=cont_cal_fit_tag_eff_corr_info,
    reco_mode_column_name=TagRecoModeInfos.fei_reco_mode_col_name,
    component_id_column_name=fei_tag_b_reco_based_fit_component_id_column_name,
    component_id_function=add_fei_tag_b_reco_based_fit_component_id,
)


cont_cal_fit_setup: FitSetupTriplet = FitSetupTriplet(asimov=_cont_cal_asimov_fit_setup)
