"""
Fit setup info for Continuum calibration fit.

Felix Metzner 2023
"""

from typing import Tuple
from uncertainties import ufloat

from templatefitter.plotter.plot_style import TangoColors

from rdstar.utilities.logging import log_to_default

from rdstar.latex_labels.latex_process_labels import LatexDecayProcesses

from rdstar.offline_analysis.observables.common_observables import Cols
from rdstar.offline_analysis.reco_modes.reco_mode_infos import RecoModeInfos
from rdstar.offline_analysis.mc_matching.old_mc_matching_info import OldMCMatchingIDsQueryInfo
from rdstar.offline_analysis.mc_matching.mc_matching_id_collections import MCMatchingCollection, MCMatchingCollectionItem

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups.fit_setup_utilities import (
    FitSetupTriplet,
    base_asimov_data_setup_info,
    cont_cal_b_charge_fit_obs_setup,
    base_cont_cal_weight_info,
    cont_cal_bbbar_bkg_sig_ids,
    common_cont_cal_fit_component_id_column_name,
    add_common_cont_cal_fit_component_id_to,
    get_sig_ids_with_charge_info,
)
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_info_container import (
    FitSetupInfoContainer,
    ComponentInfo,
    TagEfficiencyCorrectionInfo,
    RecoChannelInfo,
)


__all__ = [
    "cont_cal_fit_setup",
    "continuum_cal_fit__component_info_collection",
]


logger = log_to_default(name=__name__)


# Definition of components:

# (Main) Continuum Components

component_charged_b_reco_continuum_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    reco_type="charged",
    charge_type=None,
)
component_continuum_charged_b: ComponentInfo = ComponentInfo(
    name="continuum_in_charged_b",
    comp_index=10,
    latex_label=LatexDecayProcesses.to_continuum_txt + " in " + LatexDecayProcesses.reco_bpm_to_d_wwos,
    sig_ids=component_charged_b_reco_continuum_ids,
    color=TangoColors.plum1,
)


component_neutral_b_reco_continuum_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    reco_type="neutral",
    charge_type=None,
)
component_continuum_neutral_b: ComponentInfo = ComponentInfo(
    name="continuum_in_neutral_b",
    comp_index=11,
    latex_label=LatexDecayProcesses.to_continuum_txt + " in " + LatexDecayProcesses.reco_bz_to_d_wwos,
    sig_ids=component_neutral_b_reco_continuum_ids,
    color=TangoColors.plum3,
)


# Background Components

# BKG in Charged B

cont_cal_charged_b_d_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.Dl.id,
    reco_type="charged",
    charge_type=None,
    cross_feed_type="wwo_cf",
)

cont_cal_charged_b_ds_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.DSl.id,
    reco_type="charged",
    charge_type=None,
    cross_feed_type="wwo_cf",
)

cont_cal_charged_b_bbbar_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=cont_cal_bbbar_bkg_sig_ids,
    reco_type="charged",
    charge_type=None,
)

cont_cal_charged_b_bkg_ids: Tuple[int, ...] = (
    *cont_cal_charged_b_d_bkg_ids,
    *cont_cal_charged_b_ds_bkg_ids,
    *cont_cal_charged_b_bbbar_bkg_ids,
)

component_charged_b_bkg: ComponentInfo = ComponentInfo(
    name="charged_b_BKG",
    comp_index=20,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg + " in " + LatexDecayProcesses.reco_bpm_to_d_wwos,
    sig_ids=cont_cal_charged_b_bkg_ids,
    color=TangoColors.aluminium3,
)


# BKG in Neutral B

cont_cal_neutral_b_d_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.Dl.id,
    reco_type="neutral",
    charge_type=None,
    cross_feed_type="wwo_cf",
)

cont_cal_neutral_b_ds_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.DSl.id,
    reco_type="neutral",
    charge_type=None,
    cross_feed_type="wwo_cf",
)

cont_cal_neutral_b_bbbar_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=cont_cal_bbbar_bkg_sig_ids,
    reco_type="neutral",
    charge_type=None,
)

cont_cal_neutral_b_bkg_ids: Tuple[int, ...] = (
    *cont_cal_neutral_b_d_bkg_ids,
    *cont_cal_neutral_b_ds_bkg_ids,
    *cont_cal_neutral_b_bbbar_bkg_ids,
)

component_neutral_b_bkg: ComponentInfo = ComponentInfo(
    name="neutral_b_BKG",
    comp_index=21,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg + " in " + LatexDecayProcesses.reco_bz_to_d_wwos,
    sig_ids=cont_cal_neutral_b_bkg_ids,
    color=TangoColors.aluminium5,
)


cont_cal_fit_comps: Tuple[ComponentInfo, ...] = (
    # (Main) Continuum Components
    component_continuum_charged_b,
    component_continuum_neutral_b,
    # (Bkg) BBbar Components
    component_charged_b_bkg,
    component_neutral_b_bkg,
)


__sig_id__continuum_cal_fit__collection_definition: Tuple[MCMatchingCollectionItem, ...] = tuple(
    [
        MCMatchingCollectionItem(
            name=comp.name,
            sig_ids=comp.sig_ids,
            new_id=comp.comp_index,
            latex_label=comp.latex_label,
            color=comp.color,
        )
        for comp in cont_cal_fit_comps
    ]
)
continuum_cal_fit__component_info_collection: MCMatchingCollection = MCMatchingCollection(
    collection_name="original_sig_id__continuum_cal_fit",
    sig_id_col_name="__tmp__" + common_cont_cal_fit_component_id_column_name,
    original_sig_id_col_name=common_cont_cal_fit_component_id_column_name,
    collection_definition=__sig_id__continuum_cal_fit__collection_definition,
    base_mc_matching_id_query_infos=OldMCMatchingIDsQueryInfo,
    is_reduced_subset_of_original=False,
)


cont_cal_fit_chs: Tuple[RecoChannelInfo, ...] = tuple(
    [
        RecoChannelInfo(
            name="neutralB",
            ch_index=0,
            reco_mode_info=RecoModeInfos.neutral_b_to_d_wwo_star_modes,
            components=cont_cal_fit_comps,
            observable_infos=cont_cal_b_charge_fit_obs_setup.cont_obs_info_neutral_b,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
        RecoChannelInfo(
            name="chargedB",
            ch_index=1,
            reco_mode_info=RecoModeInfos.charged_b_to_d_wwo_star_modes,
            components=cont_cal_fit_comps,
            observable_infos=cont_cal_b_charge_fit_obs_setup.cont_obs_info_charged_b,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
    ]
)


cont_cal_fit_tag_eff_corr_info: TagEfficiencyCorrectionInfo = TagEfficiencyCorrectionInfo(
    items=[],
    key_order=(),
)


_cont_cal_asimov_fit_setup = FitSetupInfoContainer(
    components=cont_cal_fit_comps,
    reco_channels=cont_cal_fit_chs,
    observable_infos=cont_cal_b_charge_fit_obs_setup.cont_cal_obs_infos,
    weight_info=base_cont_cal_weight_info,
    data_setup_info=base_asimov_data_setup_info,
    remove_empty_bins=True,
    use_alternative_gap=True,
    tag_efficiency_correction_info=cont_cal_fit_tag_eff_corr_info,
    reco_mode_column_name=Cols.decay_mode_column_name,
    component_id_column_name=common_cont_cal_fit_component_id_column_name,
    component_id_function=add_common_cont_cal_fit_component_id_to,
)


cont_cal_fit_setup: FitSetupTriplet = FitSetupTriplet(asimov=_cont_cal_asimov_fit_setup)
