"""
Fit observable infos for fit setups of dedicated tagging efficiency calibration fit.

Felix Metzner 2022
"""

import numpy as np

from typing import Tuple

from rdstar.utilities.logging import log_to_default

from rdstar.offline_analysis.histogram_variables import RDStarHistVars

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import rdstar_fit_obs_setups
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_info_container import FitObservableInfo


__all__ = [
    "TagCalObsSetup1",
    "TagCalObsSetup1p1",
    "TagCalObsSetup1p2",
    "TagCalObsSetup2",
    "TagCalObsSetup2p1",
    "TagCalObsSetup2p2",
    "TagCalObsSetup3",
    "TagCalObsSetup3p1",
    "TagCalObsSetup3p2",
    "TagCalObsSetup3p3",
    "TagCalObsSetup4",
    "TagCalObsSetup5",
    "TagCalObsSetup5p1",
    "TagCalObsSetup5p2",
    "TagCalPerBTagModeObsSetup1",
    "TagCalPerBTagModeObsSetup2",
]


logger = log_to_default(name=__name__)


class TagCalObsSetup1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_for_main_fei_cal,
        alt_scope=None,
        alt_binning=None,
    )
    obs2 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=None,
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (
        obs1,
        obs2,
    )

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=10),
        obs2.reset(scope=None, binning=10),
    )
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=10),
        obs2.reset(scope=None, binning=10),
    )
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=10),
        obs2.reset(scope=None, binning=10),
    )
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=10),
        obs2.reset(scope=None, binning=10),
    )


class TagCalObsSetup1p1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_res_corr_q2sb_wide,
        alt_scope=None,
        alt_binning=None,
    )
    obs2 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=tuple(np.linspace(*RDStarHistVars.pl_star_signal_fit.scope, 8)),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (
        obs1,
        obs2,
    )

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=8),
        obs2.reset(scope=None, binning=None),
    )
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=12),
        obs2.reset(scope=None, binning=None),
    )
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=6),
        obs2.reset(scope=None, binning=None),
    )
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=10),
        obs2.reset(scope=None, binning=None),
    )


class TagCalObsSetup1p2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_res_corr_q2sb_wide,
        alt_scope=None,
        alt_binning=None,
    )
    obs2 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=tuple(np.linspace(*RDStarHistVars.pl_star_signal_fit.scope, 16)),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (
        obs1,
        obs2,
    )

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=20),
        obs2.reset(scope=None, binning=None),
    )
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=28),
        obs2.reset(scope=None, binning=None),
    )
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=16),
        obs2.reset(scope=None, binning=None),
    )
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=20),
        obs2.reset(scope=None, binning=None),
    )


class TagCalObsSetup2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_res_corr_q2sb_wide,
        alt_scope=None,
        alt_binning=None,
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=10),)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=9),)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)


class TagCalObsSetup2p1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_res_corr_q2sb_wide,
        alt_scope=None,
        alt_binning=None,
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=8),)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=12),)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=6),)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=10),)


class TagCalObsSetup2p2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_for_main_fei_cal,
        alt_scope=None,
        alt_binning=None,
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=10),)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=10),)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=10),)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=10),)


class TagCalObsSetup3:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=None,
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1,)


class TagCalObsSetup3p1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=tuple(np.linspace(*RDStarHistVars.pl_star_signal_fit.scope, 12)),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1,)


class TagCalObsSetup3p2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=tuple(np.linspace(*RDStarHistVars.pl_star_signal_fit.scope, 26)),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1,)


class TagCalObsSetup3p3:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=None,
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=28),)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=16),)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)


TagCalObsSetup4 = rdstar_fit_obs_setups.FitObsSetup4


class TagCalObsSetup5:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.neextra_for_fit,
        alt_scope=None,
        alt_binning=(0.0, 0.04, 0.8, 0.12, 0.16, 0.2, 0.3, 0.4, 0.6, 1.0, 1.5, 2.0, 2.5),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1,)


class TagCalObsSetup5p1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.neextra_for_fit,
        alt_scope=None,
        alt_binning=tuple(np.linspace(*RDStarHistVars.neextra_for_fit.scope, 12)),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1,)


class TagCalObsSetup5p2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.neextra_for_fit,
        alt_scope=None,
        alt_binning=tuple(np.linspace(*RDStarHistVars.neextra_for_fit.scope, 26)),
    )

    rdstar_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    rdstar_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1,)
    rdstar_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1,)


class TagCalPerBTagModeObsSetup1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_res_corr_q2sb_wide,
        alt_scope=None,
        alt_binning=None,
    )
    obs2 = FitObservableInfo(
        hist_var=RDStarHistVars.pl_star_signal_fit,
        alt_scope=None,
        alt_binning=None,
    )

    tag_cal_ptm_obs_infos: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )

    tag_cal_ptm_obs_info_d_1cpi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )
    tag_cal_ptm_obs_info_d_mcpi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )
    tag_cal_ptm_obs_info_ocd_pipi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )
    tag_cal_ptm_obs_info_ds_1cpi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )
    tag_cal_ptm_obs_info_ds_mcpi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )
    tag_cal_ptm_obs_info_j_psi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=7),
        obs2.reset(scope=None, binning=7),
    )
    tag_cal_ptm_obs_info_all_dd: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=9),
        obs2.reset(scope=None, binning=9),
    )


class TagCalPerBTagModeObsSetup2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_res_corr_q2sb_wide,
        alt_scope=None,
        alt_binning=None,
    )

    tag_cal_ptm_obs_infos: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)

    tag_cal_ptm_obs_info_d_1cpi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
    tag_cal_ptm_obs_info_d_mcpi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
    tag_cal_ptm_obs_info_ocd_pipi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
    tag_cal_ptm_obs_info_ds_1cpi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
    tag_cal_ptm_obs_info_ds_mcpi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
    tag_cal_ptm_obs_info_j_psi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=9),)
    tag_cal_ptm_obs_info_all_dd: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=11),)
