"""
Fit setup info for dedicated tagging efficiency calibration fit.

Felix Metzner 2022
"""

from typing import Tuple
from uncertainties import ufloat

from rdstar.utilities.logging import log_to_default

from rdstar.latex_labels.latex_process_labels import LatexDecayProcesses

from rdstar.offline_analysis.observables.common_observables import Cols
from rdstar.offline_analysis.reco_modes.reco_mode_infos import RecoModeInfos
from rdstar.offline_analysis.mc_matching.old_mc_matching_info import OldMCMatchingIDsQueryInfo

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups.fit_setup_utilities import (
    FitSetupTriplet,
    base_asimov_data_setup_info,
    tag_cal_fit_obs_setup,
    base_tag_cal_weight_info,
    tag_cal_bbbar_bkg_sig_ids,
    tag_cal_dstst_l_bkg_sig_ids,
    common_tag_cal_fit_component_id_column_name,
    add_common_tag_cal_fit_component_id_to,
    get_sig_ids_with_charge_info,
)
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_info_container import (
    FitSetupInfoContainer,
    ComponentInfo,
    TagEfficiencyCorrectionInfo,
    RecoChannelInfo,
)


__all__ = [
    "tag_cal_fit_setup",
]


logger = log_to_default(name=__name__)


# Definition of components:

# (Main) D* Components
component_Bp_DzS_l_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.DSl.id,
    reco_type="charged",
    charge_type=None,
    cross_feed_type="wo_cf",
)
component_Bp_DzS_l: ComponentInfo = ComponentInfo(
    name="BpDzSl",
    comp_index=40,
    latex_label=LatexDecayProcesses.to_dz_st_l_nu,
    sig_ids=component_Bp_DzS_l_ids,
)


component_Bz_DmS_l_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.DSl.id,
    reco_type="neutral",
    charge_type=None,
    cross_feed_type="wo_cf",
)
component_Bz_DmS_l: ComponentInfo = ComponentInfo(
    name="BzDmSl",
    comp_index=41,
    latex_label=LatexDecayProcesses.to_dm_st_l_nu,
    sig_ids=component_Bz_DmS_l_ids,
)


# (Main) D Components
component_Bp_Dz_l_base_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.Dl.id,
    reco_type="charged",
    charge_type=None,
    cross_feed_type="wo_cf",
)
component_Bp_Dz_l_ids: Tuple[int, ...] = (*component_Bp_Dz_l_base_ids,)
component_Bp_Dz_l: ComponentInfo = ComponentInfo(
    name="BpDzl",
    comp_index=30,
    latex_label=LatexDecayProcesses.to_dz_l_nu,
    sig_ids=component_Bp_Dz_l_ids,
)


component_Bz_Dm_l_base_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.Dl.id,
    reco_type="neutral",
    charge_type=None,
    cross_feed_type="wo_cf",
)
component_Bz_Dm_l_ids: Tuple[int, ...] = (*component_Bz_Dm_l_base_ids,)
component_Bz_Dm_l: ComponentInfo = ComponentInfo(
    name="BzDml",
    comp_index=31,
    latex_label=LatexDecayProcesses.to_dm_l_nu,
    sig_ids=component_Bz_Dm_l_ids,
)


# Feed Down D* Components
component_Bp_DzS_l_df: ComponentInfo = ComponentInfo(
    name="BpDzSlDF",
    comp_index=101,
    latex_label=LatexDecayProcesses.to_dz_st_l_nu,
    sig_ids=get_sig_ids_with_charge_info(
        sig_ids=OldMCMatchingIDsQueryInfo.DSl.id,
        reco_type="charged",
        charge_type=None,
        cross_feed_type="cf",
    ),
)

component_Bz_DmS_l_df: ComponentInfo = ComponentInfo(
    name="BzDmSlDF",
    comp_index=102,
    latex_label=LatexDecayProcesses.to_dm_st_l_nu,
    sig_ids=get_sig_ids_with_charge_info(
        sig_ids=OldMCMatchingIDsQueryInfo.DSl.id,
        reco_type="neutral",
        charge_type=None,
        cross_feed_type="cf",
    ),
)


# Background Components
tag_cal_charged_d_bbbar_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_bbbar_bkg_sig_ids,
    reco_type="c_d",
    charge_type=None,
)
component_charged_d_recoB_Dss_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_dstst_l_bkg_sig_ids,
    reco_type="c_d",
    charge_type=None,
)
component_charged_d_reco_continuum_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    reco_type="c_d",
    charge_type=None,
)


tag_cal_charged_d_bkg_ids: Tuple[int, ...] = (
    *tag_cal_charged_d_bbbar_bkg_ids,
    *component_charged_d_recoB_Dss_ids,
    *component_charged_d_reco_continuum_bkg_ids,
)

component_charged_d_bkg: ComponentInfo = ComponentInfo(
    name="charged_d_BKG",
    comp_index=60,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg,
    sig_ids=tag_cal_charged_d_bkg_ids,
)


tag_cal_neutral_d_bbbar_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_bbbar_bkg_sig_ids,
    reco_type="n_d",
    charge_type=None,
)
component_neutral_d_recoB_Dss_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_dstst_l_bkg_sig_ids,
    reco_type="n_d",
    charge_type=None,
)
component_neutral_d_reco_continuum_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    reco_type="n_d",
    charge_type=None,
)

tag_cal_neutral_d_bkg_ids: Tuple[int, ...] = (
    *tag_cal_neutral_d_bbbar_bkg_ids,
    *component_neutral_d_recoB_Dss_ids,
    *component_neutral_d_reco_continuum_bkg_ids,
)

component_neutral_d_bkg: ComponentInfo = ComponentInfo(
    name="neutral_d_BKG",
    comp_index=61,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg,
    sig_ids=tag_cal_neutral_d_bkg_ids,
)


tag_cal_charged_ds_bbbar_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_bbbar_bkg_sig_ids,
    reco_type="c_dst",
    charge_type=None,
)
component_charged_ds_recoB_Dss_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_dstst_l_bkg_sig_ids,
    reco_type="c_dst",
    charge_type=None,
)
component_charged_ds_reco_continuum_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    reco_type="c_dst",
    charge_type=None,
)
component_Bp_Dz_l_cf_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.Dl.id,
    reco_type="charged",
    charge_type=None,
    cross_feed_type="cf",
)

tag_cal_charged_ds_bkg_ids: Tuple[int, ...] = (
    *tag_cal_charged_ds_bbbar_bkg_ids,
    *component_charged_ds_recoB_Dss_ids,
    *component_charged_ds_reco_continuum_bkg_ids,
    *component_Bp_Dz_l_cf_ids,
)

component_charged_ds_bkg: ComponentInfo = ComponentInfo(
    name="charged_dst_BKG",
    comp_index=62,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg,
    sig_ids=tag_cal_charged_ds_bkg_ids,
)


tag_cal_neutral_ds_bbbar_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_bbbar_bkg_sig_ids,
    reco_type="n_dst",
    charge_type=None,
)
component_neutral_ds_recoB_Dss_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=tag_cal_dstst_l_bkg_sig_ids,
    reco_type="n_dst",
    charge_type=None,
)
component_neutral_ds_reco_continuum_bkg_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.CBKG.id,
    reco_type="n_dst",
    charge_type=None,
)
component_Bz_Dm_l_cf_ids: Tuple[int, ...] = get_sig_ids_with_charge_info(
    sig_ids=OldMCMatchingIDsQueryInfo.Dl.id,
    reco_type="neutral",
    charge_type=None,
    cross_feed_type="cf",
)

tag_cal_neutral_ds_bkg_ids: Tuple[int, ...] = (
    *tag_cal_neutral_ds_bbbar_bkg_ids,
    *component_neutral_ds_recoB_Dss_ids,
    *component_neutral_ds_reco_continuum_bkg_ids,
    *component_Bz_Dm_l_cf_ids,
)

component_neutral_ds_bkg: ComponentInfo = ComponentInfo(
    name="neutral_dst_BKG",
    comp_index=63,
    latex_label=LatexDecayProcesses.to_b_bbar_bkg,
    sig_ids=tag_cal_neutral_ds_bkg_ids,
)


tag_cal_fit_comps: Tuple[ComponentInfo, ...] = (
    component_Bp_Dz_l,
    component_Bz_Dm_l,
    component_Bp_DzS_l,
    component_Bz_DmS_l,
    component_charged_d_bkg,
    component_neutral_d_bkg,
    component_charged_ds_bkg,
    component_neutral_ds_bkg,
    component_Bp_DzS_l_df,
    component_Bz_DmS_l_df,
)


tag_cal_fit_chs: Tuple[RecoChannelInfo, ...] = tuple(
    [
        RecoChannelInfo(
            name="Bz_to_Dm",
            ch_index=0,
            reco_mode_info=RecoModeInfos.neutral_b_to_d_modes,
            components=tag_cal_fit_comps,
            observable_infos=tag_cal_fit_obs_setup.rdstar_obs_info_Bz_to_Dm,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
        RecoChannelInfo(
            name="Bm_to_Dz",
            ch_index=1,
            reco_mode_info=RecoModeInfos.charged_b_to_d_modes,
            components=tag_cal_fit_comps,
            observable_infos=tag_cal_fit_obs_setup.rdstar_obs_info_Bm_to_Dz,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
        RecoChannelInfo(
            name="Bz_to_Dsm",
            ch_index=2,
            reco_mode_info=RecoModeInfos.neutral_b_to_ds_modes,
            components=tag_cal_fit_comps,
            observable_infos=tag_cal_fit_obs_setup.rdstar_obs_info_Bz_to_Dsm,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
        RecoChannelInfo(
            name="Bm_to_Dsz",
            ch_index=3,
            reco_mode_info=RecoModeInfos.charged_b_to_ds_modes,
            components=tag_cal_fit_comps,
            observable_infos=tag_cal_fit_obs_setup.rdstar_obs_info_Bm_to_Dsz,
            tag_eff_corr_factor=ufloat(0.0, 0.00),
        ),
    ]
)


tag_cal_fit_tag_eff_corr_info: TagEfficiencyCorrectionInfo = TagEfficiencyCorrectionInfo(
    items=[],
    key_order=(),
)


_tag_cal_asimov_fit_setup = FitSetupInfoContainer(
    components=tag_cal_fit_comps,
    reco_channels=tag_cal_fit_chs,
    observable_infos=tag_cal_fit_obs_setup.rdstar_obs_infos,
    weight_info=base_tag_cal_weight_info,
    data_setup_info=base_asimov_data_setup_info,
    remove_empty_bins=True,
    use_alternative_gap=True,
    tag_efficiency_correction_info=tag_cal_fit_tag_eff_corr_info,
    reco_mode_column_name=Cols.decay_mode_column_name,
    component_id_column_name=common_tag_cal_fit_component_id_column_name,
    component_id_function=add_common_tag_cal_fit_component_id_to,
)


tag_cal_fit_setup: FitSetupTriplet = FitSetupTriplet(asimov=_tag_cal_asimov_fit_setup)
