"""
Extended mc matching information.

Felix Metzner 2023
"""

from typing import Tuple, Type, Dict

from rdstar.latex_labels.latex_process_labels import LatexDecayProcesses

from rdstar.offline_analysis.observables.common_observables import Cols

from rdstar.offline_analysis.mc_matching.mc_matching_ids import MCSigIDs
from rdstar.offline_analysis.mc_matching.mc_matching_queries import MCSigQueries
from rdstar.offline_analysis.mc_matching.mc_matching_utilities import MatchQueryInfo
from rdstar.offline_analysis.mc_matching.abstract_mc_matching_info import AbstractBaseMCMatchingIDsQueryInfo


__all__ = [
    "ExtendedMCMatchingIDsQueryInfo",
]


class EMcMLL:
    # fmt: off

    # Signal
    cB_to_nD_tau_via_e_nu: str = LatexDecayProcesses.b_plus_to_d_zero_tau_plus_nu_via_e
    nB_to_cD_tau_via_e_nu: str = LatexDecayProcesses.b_zero_to_d_minus_tau_plus_nu_via_e
    cB_to_nD_tau_via_mu_nu: str = LatexDecayProcesses.b_plus_to_d_zero_tau_plus_nu_via_mu
    nB_to_cD_tau_via_mu_nu: str = LatexDecayProcesses.b_zero_to_d_minus_tau_plus_nu_via_mu
    cB_to_nDst_tau_via_e_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_plus_to_dst_zero_tau_plus_nu_via_e_with_Dst_to_pi
    nB_to_cDst_tau_via_e_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_zero_to_dst_minus_tau_plus_nu_via_e_with_Dst_to_pi
    cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_plus_to_dst_zero_tau_plus_nu_via_mu_with_Dst_to_pi
    nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_zero_to_dst_minus_tau_plus_nu_via_mu_with_Dst_to_pi
    cB_to_nDst_tau_via_e_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_plus_to_dst_zero_tau_plus_nu_via_e_with_Dst_to_photon
    nB_to_cDst_tau_via_e_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_zero_to_dst_minus_tau_plus_nu_via_e_with_Dst_to_photon
    cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_plus_to_dst_zero_tau_plus_nu_via_mu_with_Dst_to_photon
    nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_zero_to_dst_minus_tau_plus_nu_via_mu_with_Dst_to_photon
    #
    # Norm
    cB_to_nD_e_nu: str = LatexDecayProcesses.b_plus_to_d_zero_e_plus_nu
    nB_to_cD_e_nu: str = LatexDecayProcesses.b_zero_to_d_minus_e_plus_nu
    cB_to_nD_mu_nu: str = LatexDecayProcesses.b_plus_to_d_zero_mu_plus_nu
    nB_to_cD_mu_nu: str = LatexDecayProcesses.b_zero_to_d_minus_mu_plus_nu
    cB_to_nDst_e_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_plus_to_d_st_zero_e_plus_nu_with_Dst_to_pi
    nB_to_cDst_e_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_zero_to_d_st_minus_e_plus_nu_with_Dst_to_pi
    cB_to_nDst_e_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_plus_to_d_st_zero_e_plus_nu_with_Dst_to_photon
    nB_to_cDst_e_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_zero_to_d_st_minus_e_plus_nu_with_Dst_to_photon
    cB_to_nDst_mu_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_plus_to_d_st_zero_mu_plus_nu_with_Dst_to_pi
    nB_to_cDst_mu_nu_with_Dst_to_pi: str = LatexDecayProcesses.b_zero_to_d_st_minus_mu_plus_nu_with_Dst_to_pi
    cB_to_nDst_mu_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_plus_to_d_st_zero_mu_plus_nu_with_Dst_to_photon
    nB_to_cDst_mu_nu_with_Dst_to_photon: str = LatexDecayProcesses.b_zero_to_d_st_minus_mu_plus_nu_with_Dst_to_photon
    #
    # D** with pi0
    cB_to_nDone_via_pi0_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_one_zero_e_plus_nu_with_dss_via_pi0
    cB_to_nDtwostar_via_pi0_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_twostar_zero_e_plus_nu_with_dss_via_pi0
    cB_to_nDzerostar_via_pi0_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_zerostar_zero_e_plus_nu_with_dss_via_pi0
    cB_to_nDprimeone_via_pi0_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_primeone_zero_e_plus_nu_with_dss_via_pi0
    nB_to_cDone_via_pi0_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_one_minus_e_plus_nu_with_dss_via_pi0
    nB_to_cDtwostar_via_pi0_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_twostar_minus_e_plus_nu_with_dss_via_pi0
    nB_to_cDzerostar_via_pi0_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_zerostar_minus_e_plus_nu_with_dss_via_pi0
    nB_to_cDprimeone_via_pi0_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_primeone_minus_e_plus_nu_with_dss_via_pi0
    #
    cB_to_nDone_via_pi0_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_one_zero_mu_plus_nu_with_dss_via_pi0
    cB_to_nDtwostar_via_pi0_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_twostar_zero_mu_plus_nu_with_dss_via_pi0
    cB_to_nDzerostar_via_pi0_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_zerostar_zero_mu_plus_nu_with_dss_via_pi0
    cB_to_nDprimeone_via_pi0_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_primeone_zero_mu_plus_nu_with_dss_via_pi0
    nB_to_cDone_via_pi0_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_one_minus_mu_plus_nu_with_dss_via_pi0
    nB_to_cDtwostar_via_pi0_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_twostar_minus_mu_plus_nu_with_dss_via_pi0
    nB_to_cDzerostar_via_pi0_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_zerostar_minus_mu_plus_nu_with_dss_via_pi0
    nB_to_cDprimeone_via_pi0_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_primeone_minus_mu_plus_nu_with_dss_via_pi0
    #
    cB_to_nDone_via_pi0_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_one_zero_tau_plus_nu_with_dss_via_pi0
    cB_to_nDtwostar_via_pi0_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_twostar_zero_tau_plus_nu_with_dss_via_pi0
    cB_to_nDzerostar_via_pi0_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_zerostar_zero_tau_plus_nu_with_dss_via_pi0
    cB_to_nDprimeone_via_pi0_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_primeone_zero_tau_plus_nu_with_dss_via_pi0
    nB_to_cDone_via_pi0_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_one_minus_tau_plus_nu_with_dss_via_pi0
    nB_to_cDtwostar_via_pi0_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_twostar_minus_tau_plus_nu_with_dss_via_pi0
    nB_to_cDzerostar_via_pi0_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_zerostar_minus_tau_plus_nu_with_dss_via_pi0
    nB_to_cDprimeone_via_pi0_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_primeone_minus_tau_plus_nu_with_dss_via_pi0
    #
    # D** with pi+
    cB_to_nDone_via_pip_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_one_zero_e_plus_nu_with_dss_via_pip
    cB_to_nDtwostar_via_pip_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_twostar_zero_e_plus_nu_with_dss_via_pip
    cB_to_nDzerostar_via_pip_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_zerostar_zero_e_plus_nu_with_dss_via_pip
    cB_to_nDprimeone_via_pip_e_nu: str = LatexDecayProcesses.b_plus_to_dss_d_primeone_zero_e_plus_nu_with_dss_via_pip
    nB_to_cDone_via_pip_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_one_minus_e_plus_nu_with_dss_via_pip
    nB_to_cDtwostar_via_pip_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_twostar_minus_e_plus_nu_with_dss_via_pip
    nB_to_cDzerostar_via_pip_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_zerostar_minus_e_plus_nu_with_dss_via_pip
    nB_to_cDprimeone_via_pip_e_nu: str = LatexDecayProcesses.b_zero_to_dss_d_primeone_minus_e_plus_nu_with_dss_via_pip
    #
    cB_to_nDone_via_pip_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_one_zero_mu_plus_nu_with_dss_via_pip
    cB_to_nDtwostar_via_pip_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_twostar_zero_mu_plus_nu_with_dss_via_pip
    cB_to_nDzerostar_via_pip_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_zerostar_zero_mu_plus_nu_with_dss_via_pip
    cB_to_nDprimeone_via_pip_mu_nu: str = LatexDecayProcesses.b_plus_to_dss_d_primeone_zero_mu_plus_nu_with_dss_via_pip
    nB_to_cDone_via_pip_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_one_minus_mu_plus_nu_with_dss_via_pip
    nB_to_cDtwostar_via_pip_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_twostar_minus_mu_plus_nu_with_dss_via_pip
    nB_to_cDzerostar_via_pip_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_zerostar_minus_mu_plus_nu_with_dss_via_pip
    nB_to_cDprimeone_via_pip_mu_nu: str = LatexDecayProcesses.b_zero_to_dss_d_primeone_minus_mu_plus_nu_with_dss_via_pip
    cB_to_nDone_via_pip_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_one_zero_tau_plus_nu_with_dss_via_pip
    #
    cB_to_nDtwostar_via_pip_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_twostar_zero_tau_plus_nu_with_dss_via_pip
    cB_to_nDzerostar_via_pip_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_zerostar_zero_tau_plus_nu_with_dss_via_pip
    cB_to_nDprimeone_via_pip_tau_nu: str = LatexDecayProcesses.b_plus_to_dss_d_primeone_zero_tau_plus_nu_with_dss_via_pip
    nB_to_cDone_via_pip_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_one_minus_tau_plus_nu_with_dss_via_pip
    nB_to_cDtwostar_via_pip_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_twostar_minus_tau_plus_nu_with_dss_via_pip
    nB_to_cDzerostar_via_pip_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_zerostar_minus_tau_plus_nu_with_dss_via_pip
    nB_to_cDprimeone_via_pip_tau_nu: str = LatexDecayProcesses.b_zero_to_dss_d_primeone_minus_tau_plus_nu_with_dss_via_pip
    #
    # BKGs
    # Hadronic tau BKG
    cB_to_nD_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_d_zero_tau_plus_nu
    nB_to_cD_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_d_minus_tau_plus_nu
    cB_to_nDst_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_d_st_zero_tau_plus_nu
    nB_to_cDst_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_d_st_minus_tau_plus_nu
    cB_to_nDone_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_one_zero_tau_plus_nu_with_dss_via_pi0
    cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_twostar_zero_tau_plus_nu_with_dss_via_pi0
    cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_zerostar_zero_tau_plus_nu_with_dss_via_pi0
    cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_primeone_zero_tau_plus_nu_with_dss_via_pi0
    nB_to_cDone_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_one_minus_tau_plus_nu_with_dss_via_pi0
    nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_twostar_minus_tau_plus_nu_with_dss_via_pi0
    nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_zerostar_minus_tau_plus_nu_with_dss_via_pi0
    nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_primeone_minus_tau_plus_nu_with_dss_via_pi0
    cB_to_nDone_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_one_zero_tau_plus_nu_with_dss_via_pip
    cB_to_nDtwostar_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_twostar_zero_tau_plus_nu_with_dss_via_pip
    cB_to_nDzerostar_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_zerostar_zero_tau_plus_nu_with_dss_via_pip
    cB_to_nDprimeone_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_plus_to_dss_d_primeone_zero_tau_plus_nu_with_dss_via_pip
    nB_to_cDone_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_one_minus_tau_plus_nu_with_dss_via_pip
    nB_to_cDtwostar_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_twostar_minus_tau_plus_nu_with_dss_via_pip
    nB_to_cDzerostar_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_zerostar_minus_tau_plus_nu_with_dss_via_pip
    nB_to_cDprimeone_via_pip_HADRONIC_tau_nu: str = LatexDecayProcesses.hadronic_tau_b_zero_to_dss_d_primeone_minus_tau_plus_nu_with_dss_via_pip
    #
    # Hadronic Background (HBKG) -> Reconstructed lepton is no lepton
    #
    LeptonFake_HBKG: str = LatexDecayProcesses.lepton_fake_bkg
    LeptonRecoFake_HBKG: str = LatexDecayProcesses.reco_lepton_fake_bkg
    # Generic Backgrounds
    DststMesonToThree_GBKG: str = LatexDecayProcesses.b_to_dss_l_nu_with_dss_to_many_bkg
    LeptonWithoutBMesonRelation_GBKG: str = LatexDecayProcesses.lepton_without_relation_to_b_bkg
    DMesonFake_GBKG: str = LatexDecayProcesses.d_fake_bkg
    BtoDxxlxx_GBKG: str = LatexDecayProcesses.b_to_d_x_x_l_x_bkg
    LeptonFromOtherBDecay_GBKG: str = LatexDecayProcesses.lepton_from_other_b_decay_bkg
    #
    # B -> pi l nu
    NeutralPiLNu: str = LatexDecayProcesses.b_plus_to_pi_zero_l_plus_nu
    ChargedPiLNu: str = LatexDecayProcesses.b_zero_to_pi_minus_l_plus_nu
    NeutralPiTauNu: str = LatexDecayProcesses.b_plus_to_pi_zero_tau_plus_nu
    ChargedPiTauNu: str = LatexDecayProcesses.b_zero_to_pi_minus_tau_plus_nu
    #
    # Mismatch G BKGs
    LeptonCrossFeed_Tau_EToMu_GBKG: str = LatexDecayProcesses.lepton_cross_feed_tau_with_fake_mu_from_reco_e_bkg
    LeptonCrossFeed_Tau_MuToE_GBKG: str = LatexDecayProcesses.lepton_cross_feed_tau_with_fake_e_from_reco_mu_bkg
    LeptonCrossFeed_EToMu_GBKG: str = LatexDecayProcesses.lepton_cross_feed_fake_mu_from_reco_e_bkg
    LeptonCrossFeed_MuToE_GBKG: str = LatexDecayProcesses.lepton_cross_feed_fake_e_from_reco_mu_bkg
    #
    DststDaughterMismatch_nB_cDss_e_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_zero_to_dss_plus_e_bkg
    DststDaughterMismatch_nB_cDss_mu_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_zero_to_dss_plus_mu_bkg
    DststDaughterMismatch_nB_cDss_tau_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_zero_to_dss_plus_tau_bkg
    DststDaughterMismatch_cB_cDss_e_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_plus_to_dss_plus_e_bkg
    DststDaughterMismatch_cB_cDss_mu_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_plus_to_dss_plus_mu_bkg
    DststDaughterMismatch_cB_cDss_tau_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_plus_to_dss_plus_tau_bkg
    DststDaughterMismatch_nB_nDss_e_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_zero_to_dss_zero_e_bkg
    DststDaughterMismatch_nB_nDss_mu_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_zero_to_dss_zero_mu_bkg
    DststDaughterMismatch_nB_nDss_tau_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_zero_to_dss_zero_tau_bkg
    DststDaughterMismatch_cB_nDss_e_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_plus_to_dss_zero_e_bkg
    DststDaughterMismatch_cB_nDss_mu_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_plus_to_dss_zero_mu_bkg
    DststDaughterMismatch_cB_nDss_tau_GBKG: str = LatexDecayProcesses.dss_daughter_mismatch_b_plus_to_dss_zero_tau_bkg
    #
    # fmt: on
    # Generic B decays (all the events not covered by any other query)
    GBKG: str = LatexDecayProcesses.generic_bbbar_bkg
    #
    # Continuum BKG
    Charm_CBKG: str = LatexDecayProcesses.charm_continuum_bkg
    UDS_CBKG: str = LatexDecayProcesses.uds_continuum_bkg

    @classmethod
    def get_latex_label_dict(cls) -> Dict[str, str]:
        _method_names: Tuple[str, ...] = ("get_latex_label_dict",)

        return {
            attr_name: val
            for attr_name, val in cls.__dict__.items()
            if not ((attr_name in _method_names) or (attr_name.endswith("__") or attr_name.startswith("_")))
        }


class ExtendedMCMatchingIDsQueryInfo(AbstractBaseMCMatchingIDsQueryInfo):
    sig_col_name: str = Cols.extended_sig_id_col
    __required_columns: Tuple[str, ...] = MCSigQueries.required_columns
    __sig_ids_definition_class: Type[MCSigIDs] = MCSigIDs

    @classmethod
    def get_sig_col_name(cls) -> str:
        return cls.sig_col_name

    @classmethod
    def get_required_columns(cls) -> Tuple[str, ...]:
        return cls.__required_columns

    @classmethod
    def get_sig_id_class(cls) -> Type[MCSigIDs]:
        return cls.__sig_ids_definition_class

    # fmt: off
    # Signal
    cB_to_nD_tau_via_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nD_tau_via_e_nu, query=MCSigQueries.cB_to_nD_tau_via_e_nu_query, latex_label=EMcMLL.cB_to_nD_tau_via_e_nu)
    nB_to_cD_tau_via_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cD_tau_via_e_nu, query=MCSigQueries.nB_to_cD_tau_via_e_nu_query, latex_label=EMcMLL.nB_to_cD_tau_via_e_nu)
    cB_to_nD_tau_via_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nD_tau_via_mu_nu, query=MCSigQueries.cB_to_nD_tau_via_mu_nu_query, latex_label=EMcMLL.cB_to_nD_tau_via_mu_nu)
    nB_to_cD_tau_via_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cD_tau_via_mu_nu, query=MCSigQueries.nB_to_cD_tau_via_mu_nu_query, latex_label=EMcMLL.nB_to_cD_tau_via_mu_nu)
    cB_to_nDst_tau_via_e_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_pi, query=MCSigQueries.cB_to_nDst_tau_via_e_nu_with_Dst_to_pi_query, latex_label=EMcMLL.cB_to_nDst_tau_via_e_nu_with_Dst_to_pi)
    nB_to_cDst_tau_via_e_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_pi, query=MCSigQueries.nB_to_cDst_tau_via_e_nu_with_Dst_to_pi_query, latex_label=EMcMLL.nB_to_cDst_tau_via_e_nu_with_Dst_to_pi)
    cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi, query=MCSigQueries.cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi_query, latex_label=EMcMLL.cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi)
    nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi, query=MCSigQueries.nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi_query, latex_label=EMcMLL.nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi)
    cB_to_nDst_tau_via_e_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_photon, query=MCSigQueries.cB_to_nDst_tau_via_e_nu_with_Dst_to_photon_query, latex_label=EMcMLL.cB_to_nDst_tau_via_e_nu_with_Dst_to_photon)
    nB_to_cDst_tau_via_e_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_photon, query=MCSigQueries.nB_to_cDst_tau_via_e_nu_with_Dst_to_photon_query, latex_label=EMcMLL.nB_to_cDst_tau_via_e_nu_with_Dst_to_photon)
    cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon, query=MCSigQueries.cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon_query, latex_label=EMcMLL.cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon)
    nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon, query=MCSigQueries.nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon_query, latex_label=EMcMLL.nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon)
    #
    # Norm
    cB_to_nD_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nD_e_nu, query=MCSigQueries.cB_to_nD_e_nu_query, latex_label=EMcMLL.cB_to_nD_e_nu)
    nB_to_cD_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cD_e_nu, query=MCSigQueries.nB_to_cD_e_nu_query, latex_label=EMcMLL.nB_to_cD_e_nu)
    cB_to_nD_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nD_mu_nu, query=MCSigQueries.cB_to_nD_mu_nu_query, latex_label=EMcMLL.cB_to_nD_mu_nu)
    nB_to_cD_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cD_mu_nu, query=MCSigQueries.nB_to_cD_mu_nu_query, latex_label=EMcMLL.nB_to_cD_mu_nu)
    cB_to_nDst_e_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_pi, query=MCSigQueries.cB_to_nDst_e_nu_with_Dst_to_pi_query, latex_label=EMcMLL.cB_to_nDst_e_nu_with_Dst_to_pi)
    nB_to_cDst_e_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_pi, query=MCSigQueries.nB_to_cDst_e_nu_with_Dst_to_pi_query, latex_label=EMcMLL.nB_to_cDst_e_nu_with_Dst_to_pi)
    cB_to_nDst_e_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_photon, query=MCSigQueries.cB_to_nDst_e_nu_with_Dst_to_photon_query, latex_label=EMcMLL.cB_to_nDst_e_nu_with_Dst_to_photon)
    nB_to_cDst_e_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_photon, query=MCSigQueries.nB_to_cDst_e_nu_with_Dst_to_photon_query, latex_label=EMcMLL.nB_to_cDst_e_nu_with_Dst_to_photon)
    cB_to_nDst_mu_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_pi, query=MCSigQueries.cB_to_nDst_mu_nu_with_Dst_to_pi_query, latex_label=EMcMLL.cB_to_nDst_mu_nu_with_Dst_to_pi)
    nB_to_cDst_mu_nu_with_Dst_to_pi: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_pi, query=MCSigQueries.nB_to_cDst_mu_nu_with_Dst_to_pi_query, latex_label=EMcMLL.nB_to_cDst_mu_nu_with_Dst_to_pi)
    cB_to_nDst_mu_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_photon, query=MCSigQueries.cB_to_nDst_mu_nu_with_Dst_to_photon_query, latex_label=EMcMLL.cB_to_nDst_mu_nu_with_Dst_to_photon)
    nB_to_cDst_mu_nu_with_Dst_to_photon: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_photon, query=MCSigQueries.nB_to_cDst_mu_nu_with_Dst_to_photon_query, latex_label=EMcMLL.nB_to_cDst_mu_nu_with_Dst_to_photon)
    #
    # D** with pi0
    cB_to_nDone_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pi0_e_nu, query=MCSigQueries.cB_to_nDone_via_pi0_e_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pi0_e_nu)
    cB_to_nDtwostar_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pi0_e_nu, query=MCSigQueries.cB_to_nDtwostar_via_pi0_e_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pi0_e_nu)
    cB_to_nDzerostar_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pi0_e_nu, query=MCSigQueries.cB_to_nDzerostar_via_pi0_e_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pi0_e_nu)
    cB_to_nDprimeone_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pi0_e_nu, query=MCSigQueries.cB_to_nDprimeone_via_pi0_e_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pi0_e_nu)
    nB_to_cDone_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pi0_e_nu, query=MCSigQueries.nB_to_cDone_via_pi0_e_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pi0_e_nu)
    nB_to_cDtwostar_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pi0_e_nu, query=MCSigQueries.nB_to_cDtwostar_via_pi0_e_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pi0_e_nu)
    nB_to_cDzerostar_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pi0_e_nu, query=MCSigQueries.nB_to_cDzerostar_via_pi0_e_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pi0_e_nu)
    nB_to_cDprimeone_via_pi0_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pi0_e_nu, query=MCSigQueries.nB_to_cDprimeone_via_pi0_e_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pi0_e_nu)
    cB_to_nDone_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pi0_mu_nu, query=MCSigQueries.cB_to_nDone_via_pi0_mu_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pi0_mu_nu)
    cB_to_nDtwostar_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pi0_mu_nu, query=MCSigQueries.cB_to_nDtwostar_via_pi0_mu_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pi0_mu_nu)
    cB_to_nDzerostar_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pi0_mu_nu, query=MCSigQueries.cB_to_nDzerostar_via_pi0_mu_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pi0_mu_nu)
    cB_to_nDprimeone_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pi0_mu_nu, query=MCSigQueries.cB_to_nDprimeone_via_pi0_mu_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pi0_mu_nu)
    nB_to_cDone_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pi0_mu_nu, query=MCSigQueries.nB_to_cDone_via_pi0_mu_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pi0_mu_nu)
    nB_to_cDtwostar_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pi0_mu_nu, query=MCSigQueries.nB_to_cDtwostar_via_pi0_mu_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pi0_mu_nu)
    nB_to_cDzerostar_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pi0_mu_nu, query=MCSigQueries.nB_to_cDzerostar_via_pi0_mu_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pi0_mu_nu)
    nB_to_cDprimeone_via_pi0_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pi0_mu_nu, query=MCSigQueries.nB_to_cDprimeone_via_pi0_mu_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pi0_mu_nu)
    cB_to_nDone_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pi0_tau_nu, query=MCSigQueries.cB_to_nDone_via_pi0_tau_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pi0_tau_nu)
    cB_to_nDtwostar_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pi0_tau_nu, query=MCSigQueries.cB_to_nDtwostar_via_pi0_tau_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pi0_tau_nu)
    cB_to_nDzerostar_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pi0_tau_nu, query=MCSigQueries.cB_to_nDzerostar_via_pi0_tau_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pi0_tau_nu)
    cB_to_nDprimeone_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pi0_tau_nu, query=MCSigQueries.cB_to_nDprimeone_via_pi0_tau_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pi0_tau_nu)
    nB_to_cDone_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pi0_tau_nu, query=MCSigQueries.nB_to_cDone_via_pi0_tau_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pi0_tau_nu)
    nB_to_cDtwostar_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pi0_tau_nu, query=MCSigQueries.nB_to_cDtwostar_via_pi0_tau_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pi0_tau_nu)
    nB_to_cDzerostar_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pi0_tau_nu, query=MCSigQueries.nB_to_cDzerostar_via_pi0_tau_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pi0_tau_nu)
    nB_to_cDprimeone_via_pi0_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pi0_tau_nu, query=MCSigQueries.nB_to_cDprimeone_via_pi0_tau_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pi0_tau_nu)
    # D** with pi+
    cB_to_nDone_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pip_e_nu, query=MCSigQueries.cB_to_nDone_via_pip_e_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pip_e_nu)
    cB_to_nDtwostar_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pip_e_nu, query=MCSigQueries.cB_to_nDtwostar_via_pip_e_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pip_e_nu)
    cB_to_nDzerostar_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pip_e_nu, query=MCSigQueries.cB_to_nDzerostar_via_pip_e_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pip_e_nu)
    cB_to_nDprimeone_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pip_e_nu, query=MCSigQueries.cB_to_nDprimeone_via_pip_e_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pip_e_nu)
    nB_to_cDone_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pip_e_nu, query=MCSigQueries.nB_to_cDone_via_pip_e_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pip_e_nu)
    nB_to_cDtwostar_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pip_e_nu, query=MCSigQueries.nB_to_cDtwostar_via_pip_e_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pip_e_nu)
    nB_to_cDzerostar_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pip_e_nu, query=MCSigQueries.nB_to_cDzerostar_via_pip_e_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pip_e_nu)
    nB_to_cDprimeone_via_pip_e_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pip_e_nu, query=MCSigQueries.nB_to_cDprimeone_via_pip_e_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pip_e_nu)
    cB_to_nDone_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pip_mu_nu, query=MCSigQueries.cB_to_nDone_via_pip_mu_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pip_mu_nu)
    cB_to_nDtwostar_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pip_mu_nu, query=MCSigQueries.cB_to_nDtwostar_via_pip_mu_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pip_mu_nu)
    cB_to_nDzerostar_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pip_mu_nu, query=MCSigQueries.cB_to_nDzerostar_via_pip_mu_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pip_mu_nu)
    cB_to_nDprimeone_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pip_mu_nu, query=MCSigQueries.cB_to_nDprimeone_via_pip_mu_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pip_mu_nu)
    nB_to_cDone_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pip_mu_nu, query=MCSigQueries.nB_to_cDone_via_pip_mu_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pip_mu_nu)
    nB_to_cDtwostar_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pip_mu_nu, query=MCSigQueries.nB_to_cDtwostar_via_pip_mu_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pip_mu_nu)
    nB_to_cDzerostar_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pip_mu_nu, query=MCSigQueries.nB_to_cDzerostar_via_pip_mu_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pip_mu_nu)
    nB_to_cDprimeone_via_pip_mu_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pip_mu_nu, query=MCSigQueries.nB_to_cDprimeone_via_pip_mu_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pip_mu_nu)
    cB_to_nDone_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pip_tau_nu, query=MCSigQueries.cB_to_nDone_via_pip_tau_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pip_tau_nu)
    cB_to_nDtwostar_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pip_tau_nu, query=MCSigQueries.cB_to_nDtwostar_via_pip_tau_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pip_tau_nu)
    cB_to_nDzerostar_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pip_tau_nu, query=MCSigQueries.cB_to_nDzerostar_via_pip_tau_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pip_tau_nu)
    cB_to_nDprimeone_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pip_tau_nu, query=MCSigQueries.cB_to_nDprimeone_via_pip_tau_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pip_tau_nu)
    nB_to_cDone_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pip_tau_nu, query=MCSigQueries.nB_to_cDone_via_pip_tau_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pip_tau_nu)
    nB_to_cDtwostar_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pip_tau_nu, query=MCSigQueries.nB_to_cDtwostar_via_pip_tau_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pip_tau_nu)
    nB_to_cDzerostar_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pip_tau_nu, query=MCSigQueries.nB_to_cDzerostar_via_pip_tau_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pip_tau_nu)
    nB_to_cDprimeone_via_pip_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pip_tau_nu, query=MCSigQueries.nB_to_cDprimeone_via_pip_tau_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pip_tau_nu)
    #
    # BKGs
    # Hadronic tau BKG
    cB_to_nD_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nD_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nD_hadronic_tau_nu_query, latex_label=EMcMLL.cB_to_nD_HADRONIC_tau_nu)
    nB_to_cD_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cD_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cD_hadronic_tau_nu_query, latex_label=EMcMLL.nB_to_cD_HADRONIC_tau_nu)
    cB_to_nDst_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDst_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDst_hadronic_tau_nu_query, latex_label=EMcMLL.cB_to_nDst_HADRONIC_tau_nu)
    nB_to_cDst_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDst_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDst_hadronic_tau_nu_query, latex_label=EMcMLL.nB_to_cDst_HADRONIC_tau_nu)
    cB_to_nDone_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDone_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pi0_HADRONIC_tau_nu)
    cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu)
    cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu)
    cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu)
    nB_to_cDone_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDone_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pi0_HADRONIC_tau_nu)
    nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu)
    nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu)
    nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu)
    cB_to_nDone_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDone_via_pip_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDone_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDone_via_pip_HADRONIC_tau_nu)
    cB_to_nDtwostar_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDtwostar_via_pip_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDtwostar_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDtwostar_via_pip_HADRONIC_tau_nu)
    cB_to_nDzerostar_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDzerostar_via_pip_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDzerostar_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDzerostar_via_pip_HADRONIC_tau_nu)
    cB_to_nDprimeone_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.cB_to_nDprimeone_via_pip_HADRONIC_tau_nu, query=MCSigQueries.cB_to_nDprimeone_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.cB_to_nDprimeone_via_pip_HADRONIC_tau_nu)
    nB_to_cDone_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDone_via_pip_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDone_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDone_via_pip_HADRONIC_tau_nu)
    nB_to_cDtwostar_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDtwostar_via_pip_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDtwostar_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDtwostar_via_pip_HADRONIC_tau_nu)
    nB_to_cDzerostar_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDzerostar_via_pip_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDzerostar_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDzerostar_via_pip_HADRONIC_tau_nu)
    nB_to_cDprimeone_via_pip_HADRONIC_tau_nu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.nB_to_cDprimeone_via_pip_HADRONIC_tau_nu, query=MCSigQueries.nB_to_cDprimeone_via_pip_HADRONIC_tau_nu_query, latex_label=EMcMLL.nB_to_cDprimeone_via_pip_HADRONIC_tau_nu)
    #
    # Hadronic Background (HBKG) -> Reconstructed lepton is no lepton
    #
    LeptonFake_HBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonFake_HBKG, query=MCSigQueries.lepton_fake_h_bkg_query, latex_label=EMcMLL.LeptonFake_HBKG)
    LeptonRecoFake_HBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonRecoFake_HBKG, query=MCSigQueries.lepton_reco_fake_h_bkg_query, latex_label=EMcMLL.LeptonRecoFake_HBKG)
    # Generic Backgrounds
    DststMesonToThree_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststMesonToThree_GBKG, query=MCSigQueries.dstst_to_three_g_bkg_query, latex_label=EMcMLL.DststMesonToThree_GBKG)
    LeptonWithoutBMesonRelation_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonWithoutBMesonRelation_GBKG, query=MCSigQueries.lepton_but_not_from_b_g_bkg_query, latex_label=EMcMLL.LeptonWithoutBMesonRelation_GBKG)
    DMesonFake_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DMesonFake_GBKG, query=MCSigQueries.d_meson_fake_g_bkg_query, latex_label=EMcMLL.DMesonFake_GBKG)
    BtoDxxlxx_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.BtoDxxlxx_GBKG, query=MCSigQueries.b_to_d_xx_l_xx_g_bkg_query, latex_label=EMcMLL.BtoDxxlxx_GBKG)
    LeptonFromOtherBDecay_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonFromOtherBDecay_GBKG, query=MCSigQueries.lepton_from_other_b_decay_g_bkg_query, latex_label=EMcMLL.LeptonFromOtherBDecay_GBKG)
    #
    # B -> pi l nu
    NeutralPiLNu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.NeutralPiLNu, query=MCSigQueries.neutral_pi_light_lepton_nu_query, latex_label=EMcMLL.NeutralPiLNu)  # B -> pi0 l nu
    ChargedPiLNu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.ChargedPiLNu, query=MCSigQueries.charged_pi_light_lepton_nu_query, latex_label=EMcMLL.ChargedPiLNu)  # B -> pi+ l nu
    NeutralPiTauNu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.NeutralPiTauNu, query=MCSigQueries.neutral_pi_tau_nu_query, latex_label=EMcMLL.NeutralPiTauNu)  # B -> pi0 tau nu
    ChargedPiTauNu: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.ChargedPiTauNu, query=MCSigQueries.charged_pi_tau_nu_query, latex_label=EMcMLL.ChargedPiTauNu)  # B -> pi+ tau nu
    #
    # Mismatch G BKGs
    LeptonCrossFeed_Tau_EToMu_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonCrossFeed_Tau_EToMu_GBKG, query=MCSigQueries.lepton_tau_e_to_mu_crossfeed_query, latex_label=EMcMLL.LeptonCrossFeed_Tau_EToMu_GBKG)
    LeptonCrossFeed_Tau_MuToE_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonCrossFeed_Tau_MuToE_GBKG, query=MCSigQueries.lepton_tau_mu_to_e_crossfeed_query, latex_label=EMcMLL.LeptonCrossFeed_Tau_MuToE_GBKG)
    LeptonCrossFeed_EToMu_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonCrossFeed_EToMu_GBKG, query=MCSigQueries.lepton_e_to_mu_crossfeed_query, latex_label=EMcMLL.LeptonCrossFeed_EToMu_GBKG)
    LeptonCrossFeed_MuToE_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.LeptonCrossFeed_MuToE_GBKG, query=MCSigQueries.lepton_mu_to_e_crossfeed_query, latex_label=EMcMLL.LeptonCrossFeed_MuToE_GBKG)
    #
    DststDaughterMismatch_nB_cDss_e_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_nB_cDss_e_GBKG, query=MCSigQueries.dss_daughter_mismatch_nb_cdss_e_query, latex_label=EMcMLL.DststDaughterMismatch_nB_cDss_e_GBKG)
    DststDaughterMismatch_nB_cDss_mu_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_nB_cDss_mu_GBKG, query=MCSigQueries.dss_daughter_mismatch_nb_cdss_mu_query, latex_label=EMcMLL.DststDaughterMismatch_nB_cDss_mu_GBKG)
    DststDaughterMismatch_nB_cDss_tau_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_nB_cDss_tau_GBKG, query=MCSigQueries.dss_daughter_mismatch_nb_cdss_tau_query, latex_label=EMcMLL.DststDaughterMismatch_nB_cDss_tau_GBKG)
    DststDaughterMismatch_cB_cDss_e_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_cB_cDss_e_GBKG, query=MCSigQueries.dss_daughter_mismatch_cb_cdss_e_query, latex_label=EMcMLL.DststDaughterMismatch_cB_cDss_e_GBKG)
    DststDaughterMismatch_cB_cDss_mu_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_cB_cDss_mu_GBKG, query=MCSigQueries.dss_daughter_mismatch_cb_cdss_mu_query, latex_label=EMcMLL.DststDaughterMismatch_cB_cDss_mu_GBKG)
    DststDaughterMismatch_cB_cDss_tau_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_cB_cDss_tau_GBKG, query=MCSigQueries.dss_daughter_mismatch_cb_cdss_tau_query, latex_label=EMcMLL.DststDaughterMismatch_cB_cDss_tau_GBKG)
    DststDaughterMismatch_nB_nDss_e_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_nB_nDss_e_GBKG, query=MCSigQueries.dss_daughter_mismatch_nb_ndss_e_query, latex_label=EMcMLL.DststDaughterMismatch_nB_nDss_e_GBKG)
    DststDaughterMismatch_nB_nDss_mu_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_nB_nDss_mu_GBKG, query=MCSigQueries.dss_daughter_mismatch_nb_ndss_mu_query, latex_label=EMcMLL.DststDaughterMismatch_nB_nDss_mu_GBKG)
    DststDaughterMismatch_nB_nDss_tau_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_nB_nDss_tau_GBKG, query=MCSigQueries.dss_daughter_mismatch_nb_ndss_tau_query, latex_label=EMcMLL.DststDaughterMismatch_nB_nDss_tau_GBKG)
    DststDaughterMismatch_cB_nDss_e_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_cB_nDss_e_GBKG, query=MCSigQueries.dss_daughter_mismatch_cb_ndss_e_query, latex_label=EMcMLL.DststDaughterMismatch_cB_nDss_e_GBKG)
    DststDaughterMismatch_cB_nDss_mu_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_cB_nDss_mu_GBKG, query=MCSigQueries.dss_daughter_mismatch_cb_ndss_mu_query, latex_label=EMcMLL.DststDaughterMismatch_cB_nDss_mu_GBKG)
    DststDaughterMismatch_cB_nDss_tau_GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.DststDaughterMismatch_cB_nDss_tau_GBKG, query=MCSigQueries.dss_daughter_mismatch_cb_ndss_tau_query, latex_label=EMcMLL.DststDaughterMismatch_cB_nDss_tau_GBKG)
    #
    # Generic B decays (all the events not covered by any other query)
    GBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.GBKG, query=MCSigQueries.GBKG_query, latex_label=EMcMLL.GBKG)  # Other B decay
    #
    # Continuum BKG
    Charm_CBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.Charm_CBKG, query=MCSigQueries.charm_CBKG_query, latex_label=EMcMLL.Charm_CBKG)  # Charm Continuum event
    UDS_CBKG: MatchQueryInfo = MatchQueryInfo(id_info=MCSigIDs.UDS_CBKG, query=MCSigQueries.uds_CBKG_query, latex_label=EMcMLL.UDS_CBKG)  # UDS Continuum event
    # fmt: on


if __name__ == "__main__":
    # Checks for MCSigIDs
    MCSigIDs.run_checks()

    # Checks for ExtendedMCMatchingIDsQueryInfo
    ExtendedMCMatchingIDsQueryInfo.run_checks()

    for k, v in MCSigIDs.get_sig_ids_dict().items():
        print(f"{k}: {v}")

    print(MCSigIDs.get_all_sig_ids())

    print(MCSigIDs.get_reco_mode_to_sig_id_mapping())

    for mqi_key, mqi in EMcMLL.get_latex_label_dict().items():
        print(f"{mqi_key}: {mqi}")
