"""
Evaluates and visualizes information about form factor and pID correction weights
which have been added post n-tuple production.

Felix Metzner 2020
"""

import os
import pandas as pd

from abc import ABC, abstractmethod
from typing import Optional, Tuple, Dict

from analysistools.pid_corrections.pid_weight_plotter import PIDWeightsPlotter
from analysistools.pid_corrections.pid_weight_evaluator import PIDWeightEvaluation, PIDWeightInfoContainer, PIDWeightInfo

from rdstar.utilities import PathType
from rdstar.mc_reweighting.BtoDlnu.BtoDlnu_weights import BtoDFFReweighting
from rdstar.mc_reweighting.BtoDstlnu.BtoDstlnu_weights import BtoDstarFFReweighting
from rdstar.mc_reweighting.mc_reweighting_plots import plot_reweighted as plot_ff_reweighted

from rdstar.offline_analysis.weight_management.weight_information import WeightManager


__all__ = [
    "FFWeightEvaluator",
    "PIDWeightEvaluator",
    "PIDWeightEvaluationPreparer",
]


class WeightEvaluator(ABC):
    weight_reset_plot_name_extension = "withWeightReset"  # type: str

    def __init__(
        self,
        weight_key: str,
        input_df_path: PathType,
        output_dir_path: PathType,
        with_preselection: bool,
    ) -> None:
        assert weight_key in WeightManager.weight_info_keys, (weight_key, WeightManager.weight_info_keys)

        self.weight_key = weight_key  # type: str
        self.input_df_path = input_df_path  # type: PathType
        self.output_dir_path = output_dir_path  # type: PathType
        self.with_selection = with_preselection  # type: bool

    @abstractmethod
    def produce_weight_plots(self, dry_run: bool = False) -> Dict[str, PathType]:
        raise NotImplementedError()


class FFWeightEvaluator(WeightEvaluator):
    def __init__(
        self,
        weight_key: str,
        input_df_path: PathType,
        output_dir_path: PathType,
        with_preselection: bool,
    ) -> None:
        assert weight_key in WeightManager.weight_info_keys, weight_key
        assert weight_key is not WeightManager.pid_weight_info_key, weight_key
        super().__init__(
            weight_key=weight_key,
            input_df_path=input_df_path,
            output_dir_path=output_dir_path,
            with_preselection=with_preselection,
        )

        key, path = FFWeightEvaluator.get_weight_evaluation_output_key_path_pair(
            weight_key=weight_key,
            with_selection=with_preselection,
            output_dir=output_dir_path,
        )

        self.output_key = key  # type: str
        self.plot_output_path = path  # type: PathType

    def produce_weight_plots(
        self,
        dry_run: bool = False,
    ) -> Dict[str, PathType]:
        if dry_run:
            return {self.output_key: self.plot_output_path}

        os.makedirs(name=os.path.dirname(self.plot_output_path), exist_ok=True)

        assert os.path.isfile(self.input_df_path), self.input_df_path
        df = pd.read_hdf(path_or_buf=self.input_df_path)
        assert isinstance(df, pd.DataFrame), type(df).__name__

        self.produce_ff_weight_plots(df=df)

        return {self.output_key: self.plot_output_path}

    def produce_ff_weight_plots(self, df: pd.DataFrame) -> None:
        plot_ff_reweighted(
            df=df,
            output_path=self.plot_output_path,
            file_name_extension=None,
        )

        if self.weight_key == WeightManager.ff_d_weight_info_key:
            BtoDFFReweighting.finalize_weights(df=df)
        elif self.weight_key == WeightManager.ff_ds_weight_info_key:
            BtoDstarFFReweighting.finalize_weights(df=df)
        else:
            return

        plot_ff_reweighted(
            df=df,
            output_path=self.plot_output_path,
            file_name_extension=self.weight_reset_plot_name_extension,
            normal_condition=True,
        )

    @staticmethod
    def get_weight_evaluation_output_key_path_pair(
        weight_key: str,
        with_selection: bool,
        output_dir: PathType,
    ) -> Tuple[str, PathType]:
        sel_tag = WeightManager.with_selection_suffix if with_selection else ""  # type: str
        output_key = f"combined_{weight_key}{sel_tag}_plot"  # type: str
        file_name = f"{output_key}.pdf"  # type: str
        output_path = os.path.join(output_dir, file_name)  # type: PathType
        return output_key, output_path


class PIDWeightEvaluator(WeightEvaluator):
    pid_with_no_selection_sample_rate_random_seed = 13377331  # type: int

    def __init__(
        self,
        weight_key: str,
        input_df_path: PathType,
        output_dir_path: PathType,
        with_preselection: bool,
        pid_weight_key: str,
        pid_weight_info: PIDWeightInfo,
        pid_data_scaling: Optional[float],
    ) -> None:
        assert weight_key is WeightManager.pid_weight_info_key, weight_key

        super().__init__(
            weight_key=weight_key,
            input_df_path=input_df_path,
            output_dir_path=output_dir_path,
            with_preselection=with_preselection,
        )

        assert isinstance(pid_weight_info, PIDWeightInfo)
        self.pid_weight_key = pid_weight_key  # type: str
        self.pid_weight_info = pid_weight_info  # type: PIDWeightInfo
        self.pid_data_scaling = pid_data_scaling  # type: Optional[float]

        if self.pid_data_scaling is not None:
            assert 0.0 <= self.pid_data_scaling <= 1.0, self.pid_data_scaling

        sel_tag = WeightManager.with_selection_suffix if with_preselection else ""  # type: str
        self.pid_weight_plotter = PIDWeightsPlotter(
            weight_info_key=self.pid_weight_key,
            weight_info=self.pid_weight_info,
            file_name_base=f"combined_{weight_key}{sel_tag}",
            output_dir_path=self.output_dir_path,
        )

    def produce_weight_plots(
        self,
        dry_run: bool = False,
    ) -> Dict[str, PathType]:
        if dry_run:
            return self.produce_pid_weight_plots(df=pd.DataFrame(), dry_run=True)

        os.makedirs(name=os.path.dirname(self.output_dir_path), exist_ok=True)

        assert os.path.isfile(self.input_df_path), self.input_df_path
        df = pd.read_hdf(path_or_buf=self.input_df_path)
        assert isinstance(df, pd.DataFrame), type(df).__name__

        return self.produce_pid_weight_plots(df=df)

    def produce_pid_weight_plots(
        self,
        df: pd.DataFrame,
        dry_run: bool = False,
    ) -> Dict[str, PathType]:
        if self.with_selection or self.pid_data_scaling is None or dry_run:
            pid_sample_df = df
        else:
            pid_sample_df = df.sample(
                frac=self.pid_data_scaling,
                random_state=self.pid_with_no_selection_sample_rate_random_seed,
            )
            del df

        return self.pid_weight_plotter.plot_pid_weights(df=pid_sample_df, file_name_extension=None, dry_run=dry_run)


class PIDWeightEvaluationPreparer:
    def __init__(
        self,
        input_dict: Dict[str, PathType],
        output_dir_path: PathType,
        with_selection: bool,
    ) -> None:
        required_key = WeightManager.get_weight_output_key(
            weight_key=WeightManager.pid_weight_info_key,
            with_selection=with_selection,
        )
        assert required_key in input_dict.keys(), (required_key, input_dict.keys())
        path_to_pid_info_data_frame = input_dict[required_key]  # type: PathType

        self.pid_weight_evaluator = PIDWeightEvaluation(
            path_to_pid_info_data_frame=path_to_pid_info_data_frame,
            output_dir_path=output_dir_path,
            with_selection=with_selection,
        )

    def prepare_pid_infos(self) -> None:
        self.pid_weight_evaluator.prepare_weight_evaluation_data_frames(dry_run=False)

    @staticmethod
    def get_weight_info_container_output_key_path_pair(
        output_dir_path: PathType,
        with_selection: bool,
    ) -> Tuple[str, PathType]:
        return PIDWeightEvaluation.get_output_mapping_dict_output_key_path_pair(
            output_dir=output_dir_path,
            with_selection=with_selection,
        )

    @staticmethod
    def get_weight_info_container_output_key(with_selection: bool) -> str:
        return PIDWeightEvaluation.get_output_mapping_dict_output_key(with_selection=with_selection)

    @staticmethod
    def load_weight_info_container(file_path: PathType) -> PIDWeightInfoContainer:
        return PIDWeightEvaluation.load_pid_weight_infos(weight_info_collection_path=file_path)
